#!/usr/bin/env python
## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
if str(sys.path[0])=="":
    sys.path.insert(0, sys.path[1])
sys.path.insert(1,sys.path[0]+"/Core")
import Variables
try: 
    if float(sys.version[:3])<3.0: 
        reload(sys)
        sys.setdefaultencoding("utf-8")
except:pass
Variables.checkStartupVariables()
import Universals
import RoutineChecks
if RoutineChecks.checkQt4Exist():
    import Settings
    Universals.fillMySettings(False, False, False)
    Universals.isActivePyKDE4 = False
    from MyObjects import *
    import InputOutputs
    import Dialogs
    import Execute
    defaultLangCode = str(MLocale().name())
    HamsiManagerApp = MApplication(sys.argv)
    MDir.setSearchPaths("Images", MStringList(trForM(Variables.HamsiManagerDirectory+"/Themes/Default/Images/")))
    StyleFile = open(Variables.HamsiManagerDirectory+"/Themes/Default/Style.qss") 
    HamsiManagerApp.setStyleSheet(StyleFile.read())
    languageFile = MTranslator()
    if InputOutputs.isFile(Variables.HamsiManagerDirectory+"/Languages/HamsiManagerWithQt_"+defaultLangCode+".qm"):
            languageFile.load(trForM(Variables.HamsiManagerDirectory+"/Languages/HamsiManagerWithQt_"+defaultLangCode+".qm"))
    elif InputOutputs.isFile(Variables.HamsiManagerDirectory+"/Languages/HamsiManager_"+defaultLangCode+".qm"):
            languageFile.load(trForM(Variables.HamsiManagerDirectory+"/Languages/HamsiManager_"+defaultLangCode+".qm"))
    HamsiManagerApp.installTranslator(languageFile)
    MTextCodec.setCodecForCStrings(MTextCodec.codecForName("utf-8"))
    MTextCodec.setCodecForTr(MTextCodec.codecForName("utf-8"))
    HamsiManagerApp.setWindowIcon(MIcon("Images:HamsiManager-128x128.png"))
    HamsiManagerApp.setApplicationName("ConfigureHamsiManager")
    HamsiManagerApp.setApplicationVersion(Variables.version)
    HamsiManagerApp.setOrganizationDomain("hamsiapps.com")
    HamsiManagerApp.setOrganizationName("Hamsi Apps")
    activePageNo = 0
    isOnlyRoot = False
    if len(sys.argv)>1:
        if sys.argv[1]=="--configurePage":
            activePageNo = 2
        if sys.argv[1]=="--pluginPage":
            activePageNo = 3
        for argv in sys.argv:
            if argv=="--onlyRoot":
                isOnlyRoot = True
    import MyConfigure
    class Main(MMainWindow):
        def __init__(self, parent=None):
            MMainWindow.__init__(self, parent)
            myUniversals = Universals.Universals(HamsiManagerApp, self)
            Universals.fillUIUniversals()
            self.isInstallFinised = False
            self.pageNo, self.pageSize = activePageNo, 4
            self.vblMain = MVBoxLayout()
            self.hblMain = MHBoxLayout()
            self.lblLeftImage = MLabel()
            self.pmapLeftImage = MPixmap("Images:HamsiManager-128x176.png")
            self.lblLeftImage.setPixmap(self.pmapLeftImage)
            self.vblLeftColumn = MVBoxLayout()
            self.vblLeftColumn.addStretch(1)
            self.vblLeftColumn.addWidget(self.lblLeftImage)
            self.vblLeftColumn.addStretch(5)
            self.hblMain.addLayout(self.vblLeftColumn)
            self.pages = []
            for pageNo in range(self.pageSize):
                self.pages.append(self.createPage(pageNo))
                if pageNo!=self.pageNo:
                    self.pages[-1].setVisible(False)
                self.hblMain.addWidget(self.pages[-1])
            self.vblMain.addLayout(self.hblMain, 20)
            self.hblButtons = MHBoxLayout()
            self.buttons = [MPushButton(MApplication.translate("Reconfigure", "Back")), 
                            MPushButton(MApplication.translate("Reconfigure", "Forward")), 
                            MPushButton(MApplication.translate("Reconfigure", "Reconfigure"))]
            self.hblButtons.addStretch(5)
            for btnNo, btn in enumerate(self.buttons):
                if btnNo==len(self.buttons)-1 or btnNo==0:
                    btn.setVisible(False)
                self.hblButtons.addWidget(btn, 1)
                self.connect(btn,SIGNAL("clicked()"),self.pageChanged)
            self.pbtnCancel = MPushButton(MApplication.translate("Reconfigure", "Cancel"))
            self.pbtnFinish = MPushButton(MApplication.translate("Reconfigure", "Finish"))
            self.pbtnFinish.setVisible(False)
            self.hblButtons.addWidget(self.pbtnCancel, 1)
            self.hblButtons.addWidget(self.pbtnFinish, 1)
            self.connect(self.pbtnCancel,SIGNAL("clicked()"),self.close)
            self.connect(self.pbtnFinish,SIGNAL("clicked()"),self.close)
            self.vblMain.addLayout(self.hblButtons)
            self.CentralWidget = MWidget()
            self.CentralWidget.setLayout(self.vblMain)
            self.setCentralWidget(self.CentralWidget)
            self.pageChanged(True)
        
        def createPage(self, _pageNo):
            pnlPage = MWidget()
            HBox = MHBoxLayout()
            pnlPage.setLayout(HBox)
            if _pageNo==0:
                if InputOutputs.isFile(Variables.HamsiManagerDirectory+"/Languages/About_"+defaultLangCode):
                    aboutFileContent = InputOutputs.readFromFile(Variables.HamsiManagerDirectory+"/Languages/About_"+defaultLangCode, "utf-8")
                else:
                    aboutFileContent = InputOutputs.readFromFile(Variables.HamsiManagerDirectory+"/Languages/About_en_GB", "utf-8")
                lblAbout = MLabel(trForUI(aboutFileContent))
                lblAbout.setWordWrap(True)
                HBox.addWidget(lblAbout)
            elif _pageNo==1:
                if InputOutputs.isFile(Variables.HamsiManagerDirectory+"/Languages/License_"+defaultLangCode):
                    lisenceFileContent = InputOutputs.readFromFile(Variables.HamsiManagerDirectory+"/Languages/License_"+defaultLangCode, "utf-8")
                else:
                    lisenceFileContent = InputOutputs.readFromFile(Variables.HamsiManagerDirectory+"/Languages/License_en_GB", "utf-8")
                teCopying = MTextEdit()
                teCopying.setPlainText(trForUI(lisenceFileContent))
                HBox.addWidget(teCopying)
            elif _pageNo==2:
                VBox = MVBoxLayout()
                VBox.addStretch(10)
                self.isCreateDesktopShortcut = None
                self.isCreateExecutableLink = None
                if Variables.isRunningAsRoot():
                    self.isCreateExecutableLink = MCheckBox(MApplication.translate("Reconfigure", "Add To The System"))
                    self.isCreateExecutableLink.setCheckState(Mt.Checked)
                    lblExecutableLink = MLabel(MApplication.translate("Reconfigure", "Executable Link Path : "))
                    self.leExecutableLink = MLineEdit(trForM(Settings.getUniversalSetting("pathOfExecutableHamsi", "/usr/bin/hamsi")))
                    self.connect(self.isCreateExecutableLink, SIGNAL("stateChanged(int)"),self.createExecutableLinkChanged)
                    VBox.addWidget(self.isCreateExecutableLink)
                    HBox1 = MHBoxLayout()
                    HBox1.addWidget(lblExecutableLink)
                    HBox1.addWidget(self.leExecutableLink, 10)
                    VBox.addLayout(HBox1)
                else:
                    self.isCreateDesktopShortcut = MCheckBox(MApplication.translate("Reconfigure", "Create Desktop Shortcut."))
                    self.isCreateDesktopShortcut.setCheckState(Mt.Checked)
                    VBox.addWidget(self.isCreateDesktopShortcut)
                VBox.addStretch(10)
                HBox.addLayout(VBox)
            elif _pageNo==3:
                import MyPlugins
                VBox = MVBoxLayout()
                VBox.addStretch(10)
                wPlugins = MyPlugins.MyPluginsForSystem(self)
                HBox.addWidget(wPlugins)
                VBox.addStretch(10)
                HBox.addLayout(VBox)
            return pnlPage
        
        def createExecutableLinkChanged(self, _value):
            if _value==0:
                self.leExecutableLink.setEnabled(False)
            else:
                self.leExecutableLink.setEnabled(True)
            
        def pageChanged(self, _isRunningManual=False):
            if _isRunningManual==False:
                senderObject = self.sender()
                if senderObject==self.buttons[1]:
                    self.pageNo+=1
                elif senderObject==self.buttons[0]:
                    self.pageNo-=1
                elif senderObject==self.buttons[2]:
                    self.pageNo+=1
            for pageNo, pnlPage in enumerate(self.pages):
                if pageNo!=self.pageNo:
                    pnlPage.setVisible(False)
                else:
                    pnlPage.setVisible(True)
            self.buttons[0].setVisible(False)
            self.buttons[1].setVisible(False)
            self.buttons[2].setVisible(False)
            self.buttons[1].setText(MApplication.translate("Reconfigure", "Forward"))
            if self.pageNo==0:
                self.buttons[1].setVisible(True)
            elif self.pageNo==1:
                self.buttons[1].setVisible(True)
                self.buttons[1].setText(MApplication.translate("Reconfigure", "Accept"))
            elif self.pageNo==2:
                self.buttons[0].setVisible(False)
                self.buttons[1].setVisible(False)
                self.buttons[2].setVisible(True)
                self.pbtnCancel.setVisible(True)
            elif self.pageNo==3:
                self.buttons[0].setVisible(False)
                self.buttons[1].setVisible(False)
                self.buttons[2].setVisible(False)
                self.pbtnCancel.setVisible(False)
                self.pbtnFinish.setVisible(True)
                self.isInstallFinised = True
            if _isRunningManual==False:
                if senderObject==self.buttons[2]:
                    self.reConfigure()
            
        def reConfigure(self):
            oldPathOfExecutableHamsi = Settings.getUniversalSetting("pathOfExecutableHamsi", "/usr/bin/hamsi")
            if InputOutputs.isFile(Variables.HamsiManagerDirectory + "/HamsiManager.desktop"):
                MyConfigure.reConfigureFile(Variables.HamsiManagerDirectory + "/HamsiManager.desktop", Variables.HamsiManagerDirectory)
            if self.isCreateDesktopShortcut!=None:
                if self.isCreateDesktopShortcut.checkState()==Mt.Checked:
                    import Settings
                    desktopPath = Variables.getUserDesktopPath()
                    fileContent = MyConfigure.getConfiguredDesktopFileContent(Variables.HamsiManagerDirectory)
                    InputOutputs.writeToFile(desktopPath + "/HamsiManager.desktop", fileContent)
            executableLink = str(self.leExecutableLink)
            if self.isCreateExecutableLink!=None:
                if self.isCreateExecutableLink.checkState()==Mt.Checked:
                    if executableLink.strip()!="":
                        InputOutputs.createSymLink(Variables.HamsiManagerDirectory+"/HamsiManager.py", executableLink)
                        Settings.setUniversalSetting("pathOfExecutableHamsi", executableLink)
                        if oldPathOfExecutableHamsi!=executableLink:
                            if InputOutputs.isFile(oldPathOfExecutableHamsi):
                                answer = Dialogs.ask(MApplication.translate("Reconfigure", "Other Hamsi Manager Was Detected"), 
                                    str(MApplication.translate("Reconfigure", "Other Hamsi Manager executable file was detected. Are you want to delete old executable file? You can delete this old executable file : \"%s\"")) % (oldPathOfExecutableHamsi))
                                if answer!=Dialogs.Yes:
                                    InputOutputs.removeFile(oldPathOfExecutableHamsi)
                    if InputOutputs.isDir("/usr/share/applications/"):
                        fileContent = MyConfigure.getConfiguredDesktopFileContent(Variables.HamsiManagerDirectory)
                        InputOutputs.writeToFile("/usr/share/applications/HamsiManager.desktop", fileContent)
            if Variables.isRunningAsRoot()==False:
                if InputOutputs.isDir(Variables.userDirectoryPath + "/.local/applications/")==False:
                    InputOutputs.makeDirs(Variables.userDirectoryPath + "/.local/applications/")
                fileContent = MyConfigure.getConfiguredDesktopFileContent(Variables.HamsiManagerDirectory)
                InputOutputs.writeToFile(Variables.userDirectoryPath + "/.local/applications/HamsiManager.desktop", fileContent)
            MyConfigure.installKDE4Languages()
            self.isInstallFinised = True
            
        def closeEvent(self, _event):
            if self.isInstallFinised==False:
                answer = Dialogs.ask(MApplication.translate("Reconfigure", "Finalizing Configuration"), 
                            MApplication.translate("Reconfigure", "Are You Sure You Want To Quit?"))
                if answer!=Dialogs.Yes:
                    _event.ignore()
            
    if Variables.isRunningAsRoot()==False and Variables.isRunableAsRoot():
        if isOnlyRoot:
            answer = Dialogs.askSpecial(MApplication.translate("Reconfigure", "Are You Want To Run As Root?"), MApplication.translate("Reconfigure", "Hamsi Manager Configure Tool is running with user privileges.<br>Do you want to run Hamsi Manager Configure Tool with root rights?<br>"), MApplication.translate("Reconfigure", "Yes"), MApplication.translate("Reconfigure", "No (Close)"), None)
        else:
            answer = Dialogs.askSpecial(MApplication.translate("Reconfigure", "Are You Want To Run As Root?"), MApplication.translate("Reconfigure", "Hamsi Manager Configure Tool is running with user privileges.<br>Do you want to run Hamsi Manager Configure Tool with root rights?<br>"), MApplication.translate("Reconfigure", "Yes"), MApplication.translate("Reconfigure", "No (Continue as is)"), None)
        if answer==MApplication.translate("Reconfigure", "Yes"):
            myParametres = []
            if len(sys.argv)>1:
                for x in sys.argv[1:]:
                    myParametres.append(x)
            NewApp = Execute.executeReconfigureAsRoot(myParametres)
            sys.exit()
        elif isOnlyRoot:
            sys.exit()
    MainWidget=Main()
    MainWidget.setWindowTitle(MApplication.translate("Reconfigure", "Hamsi Manager Configure Tool") + " " + MApplication.applicationVersion())
    MainWidget.setGeometry(300, 300, 650, 350)
    MainWidget.show()
    Universals.isStartingSuccessfully = True
    sys.exit(HamsiManagerApp.exec_())
else:
    sys.exit()
    
    
        
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from MyObjects import *
import Universals

class MyThread(MThread):
    
    def __init__(self, action, callback=None, args=[], kwargs={}):
        MThread.__init__(self, Universals.activeWindow())
        self.action = action
        self.callback = callback
        self.args = args
        self.kwargs = kwargs
        self.data = None
        self.connect(self, SIGNAL("startedCallback"), self.startCallback)
        
    def run(self):
        if len(self.args)==0:
            self.data = self.action(**self.kwargs)
        else:
            self.data = self.action(*self.args, **self.kwargs)
        self.emit(SIGNAL("startedCallback"))
        
    def startCallback(self):
        if self.callback!=None:
            self.callback(self.data)

        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys
from os import path
from datetime import timedelta, datetime
import Variables

class Universals():
    global MainWindow, HamsiManagerApp, MySettings, setMySetting, saveSettings, isStartingSuccessfully, loggingLevel, fillMySettings, activeWindow, isShowVerifySettings, themePath, getListFromStrint, changedDefaultValuesKeys, newSettingsKeys, isCanBeShowOnMainWindow, getDateValue, isActivePyKDE4, isLoadedMyObjects, getBoolValue, windowMode, isChangeAll, isChangeSelected, tableTypesNames, tableType, getThisTableType, fillUIUniversals, clearAllChilds, threadActionState, startThreadAction, cancelThreadAction, finishThreadAction, isContinueThreadAction, printForDevelopers, isStartedCloseProcces, getStrintFromList, iconNameFormatLabels, pathOfSettingsDirectory, fileOfSettings, setPathOfSettingsDirectory, recordFilePath, translate, isRaisedAnError, trForM, trStr, trQVariant, getUtf8Data, trUnicode, trDecode, trEncode, getValue
    MainWindow = None 
    isStartingSuccessfully = False
    isStartedCloseProcces = False
    MySettings = {}
    loggingLevel = False
    isShowVerifySettings = False
    changedDefaultValuesKeys = []
    newSettingsKeys = []
    themePath = Variables.HamsiManagerDirectory + "/Themes/Default"
    isCanBeShowOnMainWindow = False
    isActivePyKDE4 = False
    isLoadedMyObjects = False
    windowMode = "Normal"
    isChangeAll = None
    isChangeSelected = None
    threadActionState = None
    tableTypesNames = ["0", "1", "2", "3", "4", "5", "6"]
    tableType = None
    iconNameFormatLabels = Variables.iconNameFormatKeys
    pathOfSettingsDirectory = Variables.userDirectoryPath+"/.HamsiApps/HamsiManager"
    fileOfSettings = "mySettings.ini"
    recordFilePath = pathOfSettingsDirectory + "/logs.txt"
    isRaisedAnError = False
    if Variables.executableHamsiManagerPath.find("HamsiManager")==-1 or Variables.executableHamsiManagerPath.find("./HamsiManager")!=-1:
        Variables.executableHamsiManagerPath = Variables.HamsiManagerDirectory + "/HamsiManager.py"
    
    def __init__(self, _app, _main):
        global MainWindow, HamsiManagerApp
        HamsiManagerApp = _app
        MainWindow = _main
    
    def setPathOfSettingsDirectory(_path):
        global pathOfSettingsDirectory
        _path = str(_path)
        if _path[-1]=="/":
            _path = _path[:-1]
        pathOfSettingsDirectory = _path
    
    def translate(_p, _s):
        try:
            return Variables.MQtGui.QApplication.translate(_p, _s)
        except:
            try:return _s.decode("utf-8")
            except: return _s
            
    def trForM(_s):
        _s = str(_s)
        return _s
        
    def trStr(_s):
        if Variables.isPython3k:
            return _s
        return _s.toString()
        
    def trUnicode(_s, _e = "utf-8"):
        if Variables.isPython3k:
            return _s
        if isinstance(_s, unicode):
            return _s
        return unicode(_s, _e)
        
    def trDecode(_s, _e = "utf-8", _p = "strict"):
        if Variables.isPython3k:
            return _s
        return _s.decode(_e, _p)
        
    def trEncode(_s, _e = "utf-8", _p = "strict"):
        if Variables.isPython3k:
            return _s
        return _s.encode(_e, _p)
        
    def getUtf8Data(_key):
        try:
            import Utf8Content
            return Utf8Content.getUtf8Data(_key)
        except Exception as err:
            printForDevelopers(str(err))
            if _key=="replacementChars":
                return {}
            else:
                if Variables.isPython3k:
                    return ""
                else:
                    return unicode("")
        
    def trQVariant(_s):
        if Variables.isPython3k:
            return _s
        return Variables.MQtCore.QVariant(_s)
        
    def fillMySettings(_setAgain=False, _isCheckUpdate=True, _isActiveKDE4=None):
        global MySettings, isShowVerifySettings, themePath, changedDefaultValuesKeys, newSettingsKeys, isActivePyKDE4, windowMode, tableType, isChangeAll, isChangeSelected
        import Settings, InputOutputs
        sets = Settings.setting()
        settingVersion = trStr(sets.value("settingsVersion"))
        defaultValues = Variables.getDefaultValues()
        valueTypesAndValues = Variables.getValueTypesAndValues()
        for keyValue in Variables.keysOfSettings:
            value = trStr(sets.value(keyValue, trQVariant(trForM(defaultValues[keyValue]))))
            if keyValue not in MySettings.keys() or _setAgain:
                MySettings[keyValue] = str(Settings.emendValue(keyValue, value, defaultValues[keyValue], valueTypesAndValues[keyValue]))
        newSettingVersion = str(MySettings["settingsVersion"])
        if _isCheckUpdate:
            if newSettingVersion!=settingVersion:
                newSettingsKeys, changedDefaultValuesKeys = Settings.updateOldSettings(settingVersion)
                isShowVerifySettings = True
        if _isActiveKDE4!=False:
            InputOutputs.fileSystemEncoding = MySettings["fileSystemEncoding"]
            if Variables.isAvailableKDE4():
                if getBoolValue("isActivePyKDE4"):
                    if Variables.isAvailablePyKDE4():
                        if isLoadedMyObjects==False:
                            isActivePyKDE4 = True
                    else:
                        MySettings["isActivePyKDE4"] = "False"
        windowMode = MySettings["windowMode"]
        themePath = Variables.HamsiManagerDirectory + "/Themes/" + MySettings["themeName"]
        if tableType == None:
            tableType = int(MySettings["tableType"])
            if tableType<0 or tableType>=len(tableTypesNames) or tableType==3:
                tableType = 1
        if isChangeAll == None:
            isChangeAll = getBoolValue("isChangeAll")
            isChangeSelected = getBoolValue("isChangeSelected")
        if getBoolValue("isInstalledKDE4Language")==False:
            import MyConfigure
            MyConfigure.installKDE4Languages()
    
    def getListFromStrint(_listString):
        listString = eval(str(_listString))
        if len(listString)==1:
            if listString[0].strip()=="":
                return []
        return listString
        
    def getStrintFromList(_list):
        listString = ""
        for x, value in enumerate(_list):
            if value!=0:
                listString += ";"
            listString += value
        return listString
        
    def getValue(_key, _valueList = None, _defaultValue = ""):
        try:
            return MySettings[_key]
        except:
            import Settings
            sets = Settings.setting()
            MySettings[_key] = str(trStr(sets.value(_key, trQVariant(trForM(_defaultValue)))))
            if _valueList != None:
                if MySettings[_key] in _valueList:
                    return MySettings[_key]
                else:
                    if _defaultValue!="":
                        MySettings[_key] = str(_defaultValue)
                        return MySettings[_key]
                    else:
                        MySettings[_key] = str(_valueList[0])
                        return MySettings[_key]
            else:
                return MySettings[_key]
    
    def getDateValue(_key):
        return datetime.strptime(MySettings[_key], "%Y %m %d %H %M %S")
    
    def getBoolValue(_key):
        if eval(MySettings[_key].title())==True:
            return True
        return False
     
    def setMySetting(_key, _value):
        global MySettings
        MySettings[_key] = str(_value)
        
    def saveSettings(_key=None):
        from Settings import setting
        sets = setting()
        if _key==None:
            keys = MySettings.keys()
        else:
            keys = [_key]
        for value in keys:
            sets.setValue(value,trQVariant(trForM(MySettings[value])))

    def activeWindow():
        from MyObjects import MApplication
        if MApplication.activeModalWidget()!=None:
            return MApplication.activeModalWidget()
        else:
            return MainWindow
        
    def getThisTableType(_tableType):
        try:
            tt = int(_tableType)
            if tt<0 or tt>=len(tableTypesNames):
                tt = 1
        except:
            try:
                for x, name in enumerate(tableTypesNames):
                    if str(name) == str(_tableType):
                        return x
                tt = 1
            except:
                    tt = 1
        return tt
        
    def fillUIUniversals():
        global tableTypesNames, iconNameFormatLabels
        from MyObjects import translate
        tableTypesNames = [translate("Tables", "Folder Table"), 
                            translate("Tables", "File Table"), 
                            translate("Tables", "Music Table"), 
                            translate("Tables", "Subfolder Table"), 
                            translate("Tables", "Cover Table"), 
                            translate("Tables", "Amarok Cover Table"), 
                            translate("Tables", "Amarok Music Table")
                            ]
        iconNameFormatLabels = [translate("Universals", "%Artist%"), 
                            translate("Universals", "%Album%"), 
                            translate("Universals", "%Year%"), 
                            translate("Universals", "%Genre%")]
        from InputOutputs import IA #For first import
            
    def clearAllChilds(_object, _isClearThis=False):
        childs = _object.children()
        for child in childs:
            clearAllChilds(child, True)
        if _isClearThis:
            try:
                _object.hide()
                _object.deleteLater()
            except:pass
                            
    def startThreadAction():
        global threadActionState
        threadActionState = True
        
    def cancelThreadAction():
        global threadActionState
        import Dialogs
        from MyObjects import translate
        answer = Dialogs.ask(translate("Universals", "Are You Sure?"),
                            translate("Universals", "Are you want to cancel these transactions?"))
        if answer==Dialogs.Yes:
            threadActionState = False
        
    def finishThreadAction():
        global threadActionState
        threadActionState = None
        
    def isContinueThreadAction():
        return threadActionState
    
    def printForDevelopers(_message):
        import logging
        if loggingLevel==logging.DEBUG:
            print (str(_message))
        
    
        
        
        
        
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os,sys
from MyObjects import *
from Viewers import ImageViewer
import Dialogs
import Organizer
import Universals

class ImageDetails(MDialog):
    global imageDialogs,closeAllImageDialogs
    imageDialogs = []
    def __init__(self, _file, _valueType="file", _isOpenDetailsOnNewWindow=True, _defaultMaxSize=[500, 400]):
        global imageDialogs
        self.defaultMaxSize = _defaultMaxSize
        if _isOpenDetailsOnNewWindow==False:
            isHasOpenedDialog=False
            for dialog in imageDialogs:
                if dialog.isVisible()==True:
                    isHasOpenedDialog=True
                    self = dialog
                    self.changeFile(_file, _valueType)
                    dialog.activateWindow()
                    dialog.raise_()
                    break
            if isHasOpenedDialog==False:
                _isOpenDetailsOnNewWindow=True
        if _isOpenDetailsOnNewWindow==True:
            imageDialogs.append(self)
            MDialog.__init__(self, MApplication.activeWindow())
            if Universals.isActivePyKDE4==True:
                self.setButtons(MDialog.NoDefault)
            self.wImage = ImageViewer.ImageViewer(self, _defaultMaxSize=_defaultMaxSize)
            self.wImage.setMinimumWidth(520)
            self.wImage.setMinimumHeight(420)
            self.changeFile(_file, _valueType)
            pbtnClose = MPushButton(translate("ImageDetails", "OK"))
            pbtnClose.setFocus()
            pbtnClose.setMaximumWidth(100)
            MObject.connect(pbtnClose, SIGNAL("clicked()"), self.close)
            HBOXs = []
            HBOXs.append(MHBoxLayout())
            HBOXs[0].addStretch(1)
            HBOXs[0].addWidget(pbtnClose)
            self.pnlMain = MWidget()
            vblMain = MVBoxLayout(self.pnlMain)
            vblMain.addWidget(self.wImage)
            vblMain.addLayout(HBOXs[0])
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(self.pnlMain)
            else:
                self.setLayout(vblMain)
            self.show()
                  
    def changeFile(self, _file, _valueType):
        if _valueType=="data":
            self.setWindowTitle(translate("ImageDetails", "Image Details"))
        else:
            self.setWindowTitle(trForUI(str(translate("ImageDetails", "Image Details ( %s )")) % (_file)))
        self.wImage.changeCoverValues(_file, _valueType)
                  
    def closeAllImageDialogs():
        for dialog in imageDialogs:
            try:
                if dialog.isVisible()==True:
                    dialog.close()
            except:
                continue



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import Variables
import InputOutputs
import Execute
from MyObjects import translate
pluginName = str(translate("MyPlugins/Krusader", "Krusader`s User Actions Menu"))
pluginVersion = "0.3"
pluginFiles = []
pluginDirectory = ""
setupDirectory = ""

def isInstallable():
    return Variables.isAvailableKDE4()

def installThisPlugin():
    try:
        myPluginStrings = [(" <action name=\"hamsimanager_Organize\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Organize With Hamsi Manager")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "You can organize with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "You can continue to edit the folder you select with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --directory %aCurrent%</command>\n"+
                    "  <defaultshortcut>Ctrl+O</defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_emendDirectory\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Auto Emend Directory")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --emendDirectory %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"),  
                    (" <action name=\"hamsimanager_emendDirectoryWithContents\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Auto Emend Directory (With Contents)")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager (With Contents)")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager (With Contents)")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --emendDirectoryWithContents %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_emendFile\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Auto Emend File")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Auto emend with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --emendFile %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_pack\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Pack It")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Pack it with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Pack it with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --pack %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_checkIcon\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Check Directory Icon")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Check directory icon with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Check directory icon with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --checkIcon %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_clearEmptyDirectories\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Clear Empty Directories")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Clear empty directories with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Clear empty directories with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --clearEmptyDirectories %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_clearUnneededs\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Clear Unneededs")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Clear unneededs with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Clear unneededs with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --clearUnneededs %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_clearIgnoreds\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Clear Ignoreds")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Clear ignoreds with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Clear ignoreds with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --clearIgnoreds %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_copyPath\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Copy Path To Clipboard")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Copy path to clipboard with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Copy path to clipboard with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --copyPath %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_fileTree\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Build File Tree")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Build file tree with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Build file tree with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --fileTree %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_removeOnlySubFiles\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Remove Sub Files")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Remove sub files with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Remove sub files with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --removeOnlySubFiles %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_pack\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Clear It")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Clear it with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Clear it with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --clear %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_hash\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Hash Digest")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Get hash digest with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Get hash digest with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --hash %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n"), 
                    (" <action name=\"hamsimanager_textCorrector\" >\n"+
                    "  <title>" + str(translate("MyPlugins/Krusader", "Correct Content")) + "</title>\n"+
                    "  <tooltip>" + str(translate("MyPlugins/Krusader", "Correct content with Hamsi Manager")) + "</tooltip>\n"+
                    "  <icon>"+Variables.HamsiManagerDirectory+"/Themes/Default/Images/HamsiManager-128x128.png</icon>\n"+
                    "  <category>Hamsi Manager</category>\n"+
                    "  <description>" + str(translate("MyPlugins/Krusader", "Correct content with Hamsi Manager")) + ".</description>\n"+
                    "  <command>python "+Variables.HamsiManagerDirectory+"/HamsiManager.py --qm --textCorrector %aCurrent%</command>\n"+
                    "  <defaultshortcut></defaultshortcut>\n"+
                    " </action>\n")]
        if Variables.isRunningAsRoot():
            destinationPath = "/usr/share/apps/krusader/"
        else:
            destinationPath = Variables.getKDE4HomePath() +"/share/apps/krusader/"
        try:
            pluginStrings = InputOutputs.IA.readFromFile(destinationPath + "useractions.xml")
        except:
            if InputOutputs.IA.isDir(destinationPath)==False:
                InputOutputs.IA.makeDirs(destinationPath)
            pluginStrings = InputOutputs.IA.readFromFile("/usr/share/apps/krusader/useraction_examples.xml")
        pluginString = ""
        for pstr in myPluginStrings:
            if pluginStrings.find(pstr.split("\n")[0])==-1:
                pluginString += pstr
        pluginStrings = pluginStrings.replace("</KrusaderUserActions>", pluginString + "</KrusaderUserActions>")
        InputOutputs.IA.writeToFile(destinationPath + "useractions.xml", pluginStrings)
        if pluginString=="":
            return "AlreadyInstalled"
    except:
        return False
    return True



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
import InputOutputs
from MyObjects import *
import Dialogs
import Organizer
import Universals
import ReportBug

class TextDetails(MDialog):
    global textDialogs, closeAllTextDialogs
    textDialogs =[]
    
    def __init__(self,_filePath,_isOpenDetailsOnNewWindow):
        global textDialogs
        if InputOutputs.IA.isFile(_filePath):
            if _isOpenDetailsOnNewWindow==False:
                isHasOpenedDialog=False
                for dialog in textDialogs:
                    if dialog.isVisible()==True:
                        isHasOpenedDialog=True
                        self = dialog
                        self.changeFile(_filePath)
                        self.activateWindow()
                        self.raise_()
                        break
                if isHasOpenedDialog==False:
                    _isOpenDetailsOnNewWindow=True
            if _isOpenDetailsOnNewWindow==True:
                textDialogs.append(self)
                MDialog.__init__(self, MApplication.activeWindow())
                if Universals.isActivePyKDE4==True:
                    self.setButtons(MDialog.NoDefault)
                self.charSet = MComboBox()
                self.charSet.addItems(Variables.getCharSets())
                self.charSet.setCurrentIndex(self.charSet.findText(Universals.MySettings["fileSystemEncoding"]))
                self.infoLabels = {}
                self.infoValues = {}
                self.fileValues = {}
                pbtnClose = MPushButton(translate("TextDetails", "Close"))
                pbtnSave = MPushButton(translate("TextDetails", "Save Changes"))
                pbtnSave.setIcon(MIcon("Images:save.png"))
                MObject.connect(pbtnClose, SIGNAL("clicked()"), self.close)
                MObject.connect(pbtnSave, SIGNAL("clicked()"), self.save)
                self.labels = [translate("TextDetails", "File Path : "), 
                                    translate("TextDetails", "Content : ")]
                self.pnlMain = MWidget()
                self.vblMain = MVBoxLayout(self.pnlMain)
                self.pnlClearable = None
                self.changeFile(_filePath, True)
                HBOXs, VBOXs = [], []
                VBOXs.append(MVBoxLayout())
                HBOXs.append(MHBoxLayout())
                HBOXs[-1].addWidget(self.charSet, 1)
                HBOXs[-1].addWidget(pbtnSave, 4)
                VBOXs[0].addLayout(HBOXs[-1])
                VBOXs[0].addWidget(pbtnClose)
                self.vblMain.addLayout(VBOXs[0], 1)
                if Universals.isActivePyKDE4==True:
                    self.setMainWidget(self.pnlMain)
                else:
                    self.setLayout(self.vblMain)
                self.show()
                self.setMinimumWidth(700)
                self.setMinimumHeight(500)
        else:
            Dialogs.showError(translate("TextDetails", "File Does Not Exist"), 
                        str(translate("TextDetails", "\"%s\" does not exist.<br>Table will be refreshed automatically!<br>Please retry.")
                            )% Organizer.getLink(trForUI(_filePath)))
            from Universals import MainWindow
            MainWindow.FileManager.makeRefresh()
    
    def changeFile(self, _filePath, _isNew=False):
        self.fileValues = InputOutputs.readTextFile(_filePath, Universals.MySettings["fileSystemEncoding"])
        self.setWindowTitle(trForUI(InputOutputs.getBaseName(self.fileValues["path"])))    
        if self.pnlClearable != None:
            Universals.clearAllChilds(self.pnlClearable, True)
        self.pnlClearable = MWidget()
        self.vblMain.insertWidget(0, self.pnlClearable, 20)
        vblClearable = MVBoxLayout(self.pnlClearable)
        self.infoLabels["path"] = MLabel(self.labels[0]) 
        self.infoLabels["content"] = MLabel(self.labels[1]) 
        dirPath = InputOutputs.getDirName(self.fileValues["path"])
        baseName = InputOutputs.getBaseName(self.fileValues["path"])
        self.infoValues["path"] = MLineEdit(trForUI(dirPath + "/" + Organizer.emend(baseName, "file")))
        self.infoValues["content"] = MPlainTextEdit(trForUI(Organizer.emend(self.fileValues["content"], "text", False, True)))
        self.infoValues["content"].setLineWrapMode(MPlainTextEdit.NoWrap)
        self.sourceCharSet = MComboBox()
        self.sourceCharSet.addItems(Variables.getCharSets())
        self.sourceCharSet.setCurrentIndex(self.sourceCharSet.findText(Universals.MySettings["fileSystemEncoding"]))
        MObject.connect(self.sourceCharSet, SIGNAL("currentIndexChanged(int)"), self.sourceCharSetChanged)
        HBOXs = []
        HBOXs.append(MHBoxLayout())
        HBOXs[-1].addWidget(self.infoLabels["path"])
        HBOXs[-1].addWidget(self.infoValues["path"])
        HBOXs[-1].addWidget(self.sourceCharSet)
        for hbox in HBOXs:
            vblClearable.addLayout(hbox)
        vblClearable.addWidget(self.infoLabels["content"])
        vblClearable.addWidget(self.infoValues["content"])
        
    def sourceCharSetChanged(self):
        try:
            self.fileValues = InputOutputs.readTextFile(self.fileValues["path"], str(self.sourceCharSet.currentText()))
            self.infoValues["content"].setPlainText(trForUI(Organizer.emend(self.fileValues["content"], "text", False, True)))
        except:
            Dialogs.showError(translate("TextDetails", "Incorrect File Encoding"), 
                        str(translate("TextDetails", "File can not decode by \"%s\" codec.<br>Please select another file encoding type.")
                            )% trForUI(self.sourceCharSet.currentText()))
    
    def closeAllTextDialogs():
        for dialog in textDialogs:
            try:
                if dialog.isVisible()==True:
                    dialog.close()
            except:
                continue
        
    def save(self):
        try:
            import Records
            Records.setTitle(translate("TextDetails", "Text File"))
            newFileValues = {}
            newFileValues["path"] = str(self.infoValues["path"].text())
            newFileValues["content"] = str(self.infoValues["content"].toPlainText())
            newPath = InputOutputs.writeTextFile(self.fileValues, newFileValues, str(self.charSet.currentText()))
            if newPath!=self.fileValues["path"]:
                self.changeFile(newPath)
            from Universals import MainWindow
            MainWindow.FileManager.makeRefresh()
            Records.saveAllRecords()
        except:
            error = ReportBug.ReportBug()
            error.show()  
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import Variables
from InputOutputs import Musics
import InputOutputs
import os
from MyObjects import *
import Dialogs
import Organizer
import time
import Universals
import ReportBug

class MusicPlayer(MWidget):
    def __init__(self, _parent, _type="bar", _file=""):
        MWidget.__init__(self, _parent)
        self.Player = None
        self.PlayerName = None
        self.file = _file
        self.type = _type
        self.tbPause = MToolButton(self)
        self.tbPause.setToolTip(translate("Player", "Pause / Continue"))
        self.tbPause.setIcon(MIcon("Images:mediaPause.png"))
        self.tbPause.setAutoRaise(True)
        #self.tbPause.setCheckable(True)
        self.tbMute = MToolButton(self)
        self.tbMute.setToolTip(translate("Player", "Mute"))
        self.tbMute.setIcon(MIcon("Images:playerMute.png"))
        self.tbMute.setAutoRaise(True)
        self.tbMute.setCheckable(True)
        self.tbPlay = MToolButton(self)
        self.tbPlay.setToolTip(translate("Player", "Play"))
        self.tbPlay.setIcon(MIcon("Images:mediaPlay.png"))
        self.tbPlay.setAutoRaise(True)
        self.tbStop = MToolButton(self)
        self.tbStop.setToolTip(translate("Player", "Stop"))
        self.tbStop.setIcon(MIcon("Images:mediaStop.png"))
        self.tbStop.setAutoRaise(True)
        self.tbPause.setEnabled(False)
        self.tbMute.setEnabled(False)
        self.tbStop.setEnabled(False)
        MObject.connect(self.tbPause, SIGNAL("clicked()"), self.pause)
        MObject.connect(self.tbMute, SIGNAL("clicked()"), self.mute)
        MObject.connect(self.tbPlay, SIGNAL("clicked()"), self.play)
        MObject.connect(self.tbStop, SIGNAL("clicked()"), self.stop)
        if _type == "bar" and Universals.windowMode==Variables.windowModeKeys[1]:
            pass
        else:
            self.info = MLabel(translate("Player", "Please Select The File You Want To Play And Click The Play Button."))
        if _type=="bar":
            #little style for bar
            self.playInBar = MToolButton(self)
            self.playInBar.setToolTip(translate("Player", "The Selected Files Are Played Here Instead Of The Details Window."))
            self.playInBar.setIcon(MIcon("Images:playInBar.png"))
            self.playInBar.setAutoRaise(True)
            self.playInBar.setCheckable(True)
            self.playInBar.setChecked(True)
            HBOXs=[]
            HBOXs.append(MHBoxLayout())
            HBOXs[0].addWidget(self.tbPause)
            HBOXs[0].addWidget(self.tbPlay)
            HBOXs[0].addWidget(self.tbStop)
            HBOXs[0].addWidget(self.tbMute)
            HBOXs[0].addWidget(self.playInBar)
            HBOXs.append(MHBoxLayout())
            if Universals.windowMode==Variables.windowModeKeys[1]:
                self.playInBar.setMaximumHeight(16)
                self.tbPause.setMaximumHeight(16)
                self.tbMute.setMaximumHeight(16)
                self.tbPlay.setMaximumHeight(16)
                self.tbStop.setMaximumHeight(16)
            else:
                HBOXs[1].addWidget(self.info)
                self.playInBar.setMinimumHeight(22)
                self.tbPause.setMinimumHeight(22)
                self.tbMute.setMinimumHeight(22)
                self.tbPlay.setMinimumHeight(22)
                self.tbStop.setMinimumHeight(22)
            VBOX = MVBoxLayout()
            VBOX.setSpacing(0)
            VBOX.addLayout(HBOXs[1])
            VBOX.addLayout(HBOXs[0])
            self.setLayout(VBOX)
            self.setMaximumSize(150, 40)
        elif _type=="dialog":
            #full style for dialog
            HBOXs=[]
            HBOXs.append(MHBoxLayout())
            HBOXs[0].addWidget(self.tbPause)
            HBOXs[0].addWidget(self.tbPlay)
            HBOXs[0].addWidget(self.tbStop)
            HBOXs[0].addWidget(self.tbMute)
            HBOXs.append(MHBoxLayout())
            HBOXs[1].addWidget(self.info)
            VBOX = MVBoxLayout()
            VBOX.setSpacing(0)
            VBOX.addLayout(HBOXs[1])
            VBOX.addLayout(HBOXs[0])
            self.setLayout(VBOX)
            MApplication.processEvents()
            self.info.setMinimumWidth(len(self.info.text())*7)
            self.tbPause.setMinimumHeight(22)
            self.tbMute.setMinimumHeight(22)
            self.tbPlay.setMinimumHeight(22)
            self.tbStop.setMinimumHeight(22)
            self.setMaximumSize(390, 44)
        if self.type != "bar" or Universals.windowMode!=Variables.windowModeKeys[1]:
            self.infoScroller = InfoScroller(self)
            self.infoScroller.start()
            
    def setInfoText(self, _info):
        if self.type == "bar" and Universals.windowMode==Variables.windowModeKeys[1]:
            Universals.MainWindow.StatusBar.showMessage(_info)
        else:
            MApplication.processEvents()
            self.info.setText(_info)
            self.info.setMinimumWidth(len(self.info.text())*7)
            
    def play(self, _filePath="", _isPlayNow=True):
        try:
            MApplication.processEvents()
            playerName = Universals.MySettings["playerName"]
            if self.Player==None or self.PlayerName != playerName:
                self.stop()
                self.PlayerName = playerName
                if playerName=="Phonon":
                    self.Player = M_Phonon()
                elif playerName=="Phonon (PySide)":
                    self.Player = M_Phonon_PySide()
                elif playerName=="tkSnack":
                    self.Player = M_tkSnack()
                else:
                    self.Player = M_MPlayer()
            self.stop()
            if _filePath=="":
                _filePath = Universals.MainWindow.Table.currentTableContentValues[Universals.MainWindow.Table.currentRow()]["path"]
            if _filePath=="" and self.file!="":
                _filePath = self.file
            else:
                self.file = _filePath
            if InputOutputs.IA.isFile(_filePath):
                self.musicTags = Musics.readMusicFile(_filePath)
                self.setInfoText(trForUI(("%s - %s (%s)") % (self.musicTags["artist"] , self.musicTags["title"], self.musicTags["album"])))
                if _isPlayNow==True:
                    if self.Player.play(_filePath):
                        self.tbPause.setEnabled(True)
                        self.tbMute.setEnabled(True)
                        self.tbStop.setEnabled(True)
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def stop(self):
        try:
            try:self.Player.stop()
            except:pass
            self.tbPause.setEnabled(False)
            self.tbMute.setEnabled(False)
            self.tbPause.setChecked(False)
            self.tbMute.setChecked(False)
            self.tbStop.setEnabled(False)
            if self.type=="bar":
                self.setInfoText(translate("Player", "Please Select The File You Want To Play And Click The Play Button."))
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def mute(self):
        try:
            if self.tbPause.isChecked():
                self.tbPause.setChecked(False)
            self.Player.mute()
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def pause(self):
        try:
            self.Player.pause()
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def runTo(self):
        try:
            temp=nesne_adi().split(";")
            self.Player.runTo(temp[1])
        except:
            error = ReportBug.ReportBug()
            error.show()
        
    def seek(self, _state):      
        try:  
            self.Player.seek(_state)
        except:
            error = ReportBug.ReportBug()
            error.show()


class M_Phonon():
    def __init__(self):
        self.m_media = None
        self.paused = False
        self.muted = False
    
    def play(self, _filePath):
        if self.m_media!=None:
           self.stop() 
        try:
            from PyQt4.phonon import Phonon
        except:
            Dialogs.showError(translate("Player", "Phonon Is Not Installed On Your System."),
                        translate("Player", "We could not find the Phonon(PyQt4) module installed on your system.<br>Please choose another player from the options or <br>check your Phonon installation."))
            return False
        import Universals
        if not self.m_media:
            self.m_media = Phonon.MediaObject(Universals.MainWindow)
            self.audioOutput = Phonon.AudioOutput(Phonon.MusicCategory, Universals.MainWindow)
            Phonon.createPath(self.m_media, self.audioOutput)
        self.m_media.setCurrentSource(
            Phonon.MediaSource(trForM(_filePath)))
        self.m_media.play()
        self.paused = False
        return True
    
    def pause(self):
        if self.paused:
            self.m_media.play()
            self.paused=False
        else:
            self.m_media.pause()
            self.paused=True
    
    def stop(self):
        if self.m_media!=None:
            self.m_media.stop()
        self.paused = False
    
    def runTo(self, _saniye):
        self.m_media.seek(_saniye)
    
    def seek(self, _state):
        pass
    
    def mute(self):
        if self.muted:
            self.audioOutput.setMuted(False)
            self.muted=False
        else:
            self.audioOutput.setMuted(True)
            self.muted=True

class M_Phonon_PySide():
    def __init__(self):
        self.m_media = None
        self.paused = False
        self.muted = False
    
    def play(self, _filePath):
        if self.m_media!=None:
           self.stop() 
        try:
            from PySide.phonon import Phonon
        except:
            Dialogs.showError(translate("Player", "Phonon Is Not Installed On Your System."),
                        translate("Player", "We could not find the Phonon(PySide) module installed on your system.<br>Please choose another player from the options or <br>check your Phonon installation."))
        import Universals
        if not self.m_media:
            self.m_media = Phonon.MediaObject()
            self.audioOutput = Phonon.AudioOutput(Phonon.MusicCategory)
            Phonon.createPath(self.m_media, self.audioOutput)
        self.m_media.setCurrentSource(Phonon.MediaSource(trForM(_filePath)))
        self.m_media.play()
        self.paused = False
        return True
    
    def pause(self):
        if self.paused:
            self.m_media.play()
            self.paused=False
        else:
            self.m_media.pause()
            self.paused=True
    
    def stop(self):
        if self.m_media!=None:
            self.m_media.stop()
        self.paused = False
    
    def runTo(self, _saniye):
        self.m_media.seek(_saniye)
    
    def seek(self, _state):
        pass
    
    def mute(self):
        if self.muted:
            self.audioOutput.setMuted(False)
            self.muted=False
        else:
            self.audioOutput.setMuted(True)
            self.muted=True

class M_tkSnack():
    def __init__(self):
        self.tada = None

    def play(self, _filePath):
        if self.tada!=None:
           self.stop() 
        from Tkinter import Tk
        import tkSnack
        self.root = Tk()
        tkSnack.initializeSnack(self.root)
        self.tada = tkSnack.Sound(file=_filePath)
        self.tada.play()
        return True

    def pause(self):
        self.tada.pause()

    def stop(self):
        if self.tada!=None:
            self.tada.stop()

    def runTo(self, _saniye):
        pass

    def seek(self, _state):
        pass

    def mute(self):
        pass

class M_MPlayer():
    
    def __init__(self):
        self.popen = False
    
    def runCommand(self, _command):
        if self.popen!=False:
            from Execute import writeToPopen
            writeToPopen(self.popen, _command)
        
    def play(self, _filePath):
        import Execute
        if self.popen!=False:
            self.runCommand("quit")
        command = [Universals.MySettings["mplayerPath"]] 
        command += Universals.MySettings["mplayerArgs"].split(" ")
        command += [Universals.MySettings["mplayerAudioDevicePointer"], 
                   Universals.MySettings["mplayerAudioDevice"], 
                   str(_filePath)]
        self.popen = Execute.execute(command)
        return True
        
    def pause(self):
        self.runCommand("pause")  
        
    def stop(self):
        self.runCommand("quit")
        try:self.popen.close()
        except:pass
        self.popen = False
        
    def runTo(self, _saniye):
        self.runCommand("seek " + str(_saniye))
    
    def seek(self, _state):
        self.runCommand("seek " + str(_state) + " 100")
    
    def mute(self):
        self.runCommand("mute")
        
class InfoScroller(MThread):
    def __init__(self, _parent):
        MThread.__init__(self)
        self.parent = _parent
    
    def run(self):
        x = 150
        while 1==1:
            if Universals!=None:
                if Universals.isStartingSuccessfully and Universals.isStartedCloseProcces==False:
                    try:
                        self.parent.info.move(x, 0)
                        time.sleep(0.05)
                        x-=1
                        self.parent.info.setMinimumWidth(len(self.parent.info.text())*7)
                        if x<=-(len(self.parent.info.text())*7):
                            x=150
                    except:pass #Passed for cleared objects
                
    



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import os
import shutil
import Variables
import Universals
import Records

class InputOutputs:
    """Read and writes are arranged in this class"""
    global isFile, isDir, moveFileOrDir, listDir, makeDirs, removeDir, removeFile, getDirName, getBaseName, copyDirTree, trSort, readDirectory, moveOrChange, moveDir, appendingDirectories, readDirectoryWithSubDirectories, clearEmptyDirectories, clearUnneededs, clearIgnoreds, checkIcon, removeFileOrDir, changeDirectories, readTextFile, writeTextFile, clearPackagingDirectory, makePack, extractPack, copyOrChange, isExist, copyDirectory, isWritableFileOrDir, getRealDirName, checkSource, checkDestination, copyFileOrDir, readDirectoryAll, getObjectType, isAvailableName, getFileExtension, readFromFile, writeToFile, addToFile, readFromBinaryFile, writeToBinaryFile, readLinesFromFile, fileSystemEncoding, clearTempFiles, getFileTree, removeOnlySubFiles, moveToPathOfDeleted, getSize, fixToSize, clearCleaningDirectory, checkExtension, isDirEmpty, createSymLink, willCheckIconDirectories, isSmartCheckIcon, activateSmartCheckIcon, completeSmartCheckIcon, setIconToDirectory, getFirstImageInDirectory, isReadableFileOrDir, getHashDigest, createHashDigestFile, getIconFromDirectory, getRealPath, getShortPath, copyDirContent
    appendingDirectories = []
    fileSystemEncoding = Variables.defaultFileSystemEncoding
    willCheckIconDirectories = []
    isSmartCheckIcon = False
    
    def isFile(_oldPath):
        _oldPath = str(_oldPath)
        try:return os.path.isfile(Universals.trEncode(_oldPath, fileSystemEncoding))
        except:return os.path.isfile(_oldPath)
    
    def isDir(_oldPath):
        _oldPath = str(_oldPath)
        try:return os.path.isdir(Universals.trEncode(_oldPath, fileSystemEncoding))
        except:return os.path.isdir(_oldPath)
    
    def isDirEmpty(_oldPath):
        _oldPath = str(_oldPath)
        if isDir(_oldPath):
            if len(listDir(_oldPath))==0:
                return True
        return False
        
    def isExist(_oldPath):
        if isFile(_oldPath):
            return True
        elif isDir(_oldPath):
            return True
        return False
        
    def isAvailableName(_newPath):
        try:
            _newPath = str(_newPath)
            t = Universals.trEncode(_newPath, fileSystemEncoding)
            return True
        except:
            return False
    
    def getSize(_oldPath):
        from stat import ST_SIZE
        try:return os.stat(Universals.trEncode(_oldPath, fileSystemEncoding))[ST_SIZE]
        except:return os.stat(_oldPath)[ST_SIZE]
    
    def getObjectType(_oldPath):
        objectType="file"
        if isDir(_oldPath):
            objectType="directory"
        return objectType
    
    def getDirName(_oldPath):
        _oldPath = str(_oldPath)
        try:returnValue = os.path.dirname(Universals.trEncode(_oldPath, fileSystemEncoding))
        except:returnValue = os.path.dirname(_oldPath)
        try:return Universals.trDecode(returnValue, fileSystemEncoding)
        except:return returnValue
    
    def getRealDirName(_oldPath, isGetParent=False):
        _oldPath = str(_oldPath)
        if len(_oldPath)==0: return "/"
        if _oldPath[-1]=="/":
            _oldPath = _oldPath[:-1]
        if isGetParent:
            realDirName = getDirName(str(_oldPath))
        else:
            realDirName = str(_oldPath)
        while 1:
            if isDir(realDirName):
                break
            if realDirName=="":
                realDirName = "/"
                break
            realDirName = getDirName(realDirName)
        return realDirName
        
    def getRealPath(_path, _parentPath=None):
        _path = str(_path)
        if len(_path)==0: return "/"
        if _parentPath!=None:
            _parentPath = getRealPath(_parentPath)
            if _path[:2]=="./":
                _path = _parentPath + _path[1:]
        return os.path.abspath(_path).replace("\\", "/")
    
    def getShortPath(_path, _parentPath):
        _path = str(_path)
        _parentPath = str(_parentPath)
        _path = _path.replace(_parentPath, ".")
        return _path
    
    def getBaseName(_oldPath):
        _oldPath = str(_oldPath)
        try:returnValue = os.path.basename(Universals.trEncode(_oldPath, fileSystemEncoding))
        except:returnValue = os.path.basename(_oldPath)
        try:return Universals.trDecode(returnValue, fileSystemEncoding)
        except:return returnValue
    
    def checkExtension(_oldPath, _extension):
        _oldPath = str(_oldPath).lower()
        _extension = str(_extension).lower()
        if _extension.strip()!="":
            if _extension[0]==".": 
                _extension = _extension[1:]
            extIndex = _oldPath.find("." + _extension)
            if extIndex!=-1:
                if _oldPath[extIndex:]== "." + _extension:
                    return True
        return False
        
    def getFileExtension(_filePath):
        _filePath = str(_filePath).lower()
        if _filePath.find(".")!=-1:
            if Universals.MySettings["fileExtesionIs"]==Variables.fileExtesionIsKeys[0]:
                return _filePath.split(".", 1)[1]
            elif Universals.MySettings["fileExtesionIs"]==Variables.fileExtesionIsKeys[1]:
                return _filePath.rsplit(".", 1)[1]
        return ""
    
    def moveFileOrDir(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        if getDirName(_oldPath)==getDirName(_newPath):
            try:os.rename(Universals.trEncode(_oldPath, fileSystemEncoding),Universals.trEncode(_newPath, fileSystemEncoding))
            except:os.rename(_oldPath,_newPath)
        else:
            if isDir(getDirName(_newPath))==False:
                makeDirs(getDirName(_newPath))
            try:shutil.move(Universals.trEncode(_oldPath, fileSystemEncoding),Universals.trEncode(_newPath, fileSystemEncoding))
            except:shutil.move(_oldPath,_newPath)
        Records.add("Moved", _oldPath, _newPath)
    
    def copyFileOrDir(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        if isDir(getDirName(_newPath))==False:
            makeDirs(getDirName(_newPath))
        if isFile(_oldPath):
            try:shutil.copy(Universals.trEncode(_oldPath, fileSystemEncoding),Universals.trEncode(_newPath, fileSystemEncoding))
            except:shutil.copy(_oldPath,_newPath)
        else:
            copyDirTree(_oldPath, _newPath)
        Records.add("Copied", _oldPath, _newPath)
            
    def copyDirTree(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        try:shutil.copytree(Universals.trEncode(_oldPath, fileSystemEncoding),Universals.trEncode(_newPath, fileSystemEncoding))
        except:shutil.copytree(_oldPath,_newPath)
        Records.add("Copied", _oldPath, _newPath)
        
    def copyDirContent(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        if isDir(_newPath)==False:
            makeDirs(_newPath)
        for contentPath in listDir(_oldPath):
            if isDir(_oldPath+"/"+contentPath):
                copyDirContent(_oldPath+"/"+contentPath, _newPath+"/"+contentPath)
            else:
                copyFileOrDir(_oldPath+"/"+contentPath, _newPath+"/"+contentPath)
    
    def createSymLink(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        if Variables.isAvailableSymLink():
            from os import symlink
            if isExist(_newPath):
                removeFile(_newPath)
            try:symlink(Universals.trEncode(_oldPath, fileSystemEncoding),Universals.trEncode(_newPath, fileSystemEncoding))
            except:symlink(_oldPath,_newPath)
            Records.add("Created Link", _oldPath, _newPath)
            return True
        else:
            Records.add("Can Not Created Link", _oldPath, _newPath)
            copyOrChange(_oldPath, _newPath, getObjectType(_oldPath))
            return False
      
    def listDir(_oldPath):
        names = []
        if checkSource(_oldPath, "directory"):
            try:names = os.listdir(Universals.trEncode(_oldPath, fileSystemEncoding))
            except:names = os.listdir(_oldPath)
            names.sort(key=trSort)
        return names
        
    def makeDirs(_newPath):
        if isWritableFileOrDir(getRealDirName(_newPath)):
            try:os.makedirs(Universals.trEncode(_newPath, fileSystemEncoding))
            except:os.makedirs(_newPath)
            Records.add("Created", _newPath)
            return True
        return False
        
    def removeDir(_oldPath):
        if Universals.getBoolValue("isDontDeleteFileAndDirectory"):
            moveToPathOfDeleted(_oldPath)
        else:
            try:os.rmdir(Universals.trEncode(_oldPath, fileSystemEncoding))
            except:os.rmdir(_oldPath)
        Records.add("Removed", _oldPath)
        return True
        
    def removeFile(_oldPath):
        if Universals.getBoolValue("isDontDeleteFileAndDirectory"):
            moveToPathOfDeleted(_oldPath)
        else:
            try:os.remove(Universals.trEncode(_oldPath, fileSystemEncoding))
            except:os.remove(_oldPath)
        Records.add("Removed", _oldPath)
        return True
    
    def moveToPathOfDeleted(_oldPath):
        from time import strftime
        import random
        moveFileOrDir(_oldPath, Universals.MySettings["pathOfDeletedFilesAndDirectories"] + "/" + strftime("%Y%m%d_%H%M%S") + "_" + str(random.randrange(0, 9999999)) + "_" + getBaseName(_oldPath))
    
    def trSort(_info):
        import locale
        if Variables.isPython3k:
            _info = str(_info)
        try:
            return locale.strxfrm(Universals.trEncode(_info, fileSystemEncoding))
        except:
            return locale.strxfrm(_info)
    
    def isReadableFileOrDir(_newPath): 
        realPath = _newPath
        if isFile(realPath)==False:
            realPath = getRealDirName(realPath)
        try: 
            if os.access(Universals.trEncode(realPath, fileSystemEncoding), os.R_OK): 
                return True 
        except: 
            if os.access(realPath, os.R_OK): 
                return True
        return False
        
    def isWritableFileOrDir(_newPath):
        realPath = _newPath
        if isFile(realPath)==False:
            realPath = getRealDirName(realPath)
        try: 
            if os.access(Universals.trEncode(realPath, fileSystemEncoding), os.W_OK): 
                return True 
        except: 
            if os.access(realPath, os.W_OK): 
                return True
        return False
        
    def checkSource(_oldPath, _objectType="fileOrDirectory"):
        if _objectType=="file" and isFile(_oldPath)==False:
            return False
        elif _objectType=="directory" and isDir(_oldPath)==False:
            return False
        elif isDir(_oldPath)==False and isFile(_oldPath)==False:
            return False
        return _oldPath
        
    def checkDestination(_oldPath, _newPath, _isMake=False):
        global appendingDirectories
        if isExist(_newPath):
            if isWritableFileOrDir(_newPath):
                if _oldPath.lower()!=_newPath.lower() or Variables.osName=="posix": 
                    if isFile(_newPath):
                        if _isMake:
                            return _newPath
                        else:
                            return False
                    elif isDir(_newPath):
                        if isFile(_oldPath):
                            return False
                        else:
                            isAllowed=False
                            for tDir in appendingDirectories:
                                if _newPath==tDir:
                                    isAllowed=True
                                    return _newPath
                            if isAllowed==False: 
                                if _isMake:
                                    appendingDirectories.append(_newPath)
                                    return _newPath
                                else:
                                    return False
                    else:
                        return False
                else:
                    return _newPath
            else:
                return False
        else:
            if isWritableFileOrDir(getDirName(_newPath)):
                return _newPath
            else:
                return False
        return False
        
    def readDirectory(_path, _objectType="fileOrDirectory"):
        global appendingDirectories
        appendingDirectories=[]
        fileAndDirectoryNames,fileNames,directoryNames,musicFileNames=[],[],[],[]
        for name in listDir(_path):
            if name[:1] != ".":
                try:fileAndDirectoryNames.append(Universals.trDecode(name, fileSystemEncoding))
                except:fileAndDirectoryNames.append(name)
        for name in fileAndDirectoryNames:
            if isDir(_path+"/"+name):
                directoryNames.append(name)
            else:
                fileNames.append(name)
                for ext in Universals.getListFromStrint(Universals.MySettings["musicExtensions"]):
                    try:
                        if name.split(".")[-1].lower() == str(ext).lower():
                            musicFileNames.append(name)
                    except:
                        pass
        fileAndDirectoryNames = []
        for d in directoryNames:
            fileAndDirectoryNames.append(d)
        for f in fileNames:
            fileAndDirectoryNames.append(f)
        if _objectType=="file":
            return fileNames
        elif _objectType=="directory":
            return directoryNames  
        elif _objectType=="fileAndDirectory":
            return fileAndDirectoryNames  
        elif _objectType=="music":
            return musicFileNames
        else:
            return []
    
    def readDirectoryAll(_path): 
        tFileAndDirs=[]
        for name in listDir(_path):
            try:tFileAndDirs.append(str(Universals.trDecode(name, fileSystemEncoding)))
            except:
                try:tFileAndDirs.append(str(name))
                except:tFileAndDirs.append(name)
        return tFileAndDirs
  
    def readDirectoryWithSubDirectories(_path, _subDirectoryDeep=-1, _isGetDirectoryNames=False, _isOnlyDirectories=False, _currentSubDeep=0):
        global appendingDirectories
        _subDirectoryDeep = int(_subDirectoryDeep)
        allFilesAndDirectories, names, files, directories, appendingDirectories =[],[],[],[],[]
        try:namesList = readDirectoryAll(_path)
        except:return []
        for name in namesList:
            if name[:1] != ".":
                names.append(name)
        for name in names:
            if isDir(_path+"/"+name):
                directories.append(name)
            else:
                files.append(name)
        names = []
        for d in directories:
            names.append(d)
        for f in files:
            names.append(f)
        for name in names:
            if name[:1] != ".":
                if isDir(_path+"/"+name):
                    if _subDirectoryDeep==-1 or _subDirectoryDeep>_currentSubDeep:
                        if _isGetDirectoryNames==True:
                            allFilesAndDirectories.append(_path+"/"+name)
                        for dd in readDirectoryWithSubDirectories(_path+"/"+name, _subDirectoryDeep, _isGetDirectoryNames, _isOnlyDirectories, _currentSubDeep+1):
                            allFilesAndDirectories.append(dd)
                elif _isOnlyDirectories==False:
                    allFilesAndDirectories.append(_path+"/"+name)
        return allFilesAndDirectories
    
    def readFromFile(_path, _contentEncoding = fileSystemEncoding):
        _path = str(_path)
        if Variables.isPython3k:
            try:f = open(Universals.trEncode(_path, fileSystemEncoding) , encoding = _contentEncoding)
            except:f = open(_path , encoding = _contentEncoding)
        else:
            import codecs
            try:f = codecs.open(Universals.trEncode(_path, fileSystemEncoding) , encoding = _contentEncoding)
            except:f = codecs.open(_path , encoding = _contentEncoding)
        info = f.read()
        f.close()
        return info
        
    def readLinesFromFile(_path, _contentEncoding = fileSystemEncoding):
        _path = str(_path)
        if Variables.isPython3k:
            try:f = open(Universals.trEncode(_path, fileSystemEncoding) , encoding = _contentEncoding)
            except:f = open(_path , encoding = _contentEncoding)
        else:
            import codecs
            try:f = codecs.open(Universals.trEncode(_path, fileSystemEncoding) , encoding = _contentEncoding)
            except:f = codecs.open(_path , encoding = _contentEncoding)
        info = f.readlines()
        f.close()
        return info
        
    def readFromBinaryFile(_path):
        _path = str(_path)
        try:f = open(Universals.trEncode(_path, fileSystemEncoding), "rb")
        except:f = open(_path, "rb")
        info = f.read()
        f.close()
        return info
        
    def writeToFile(_path, _contents=""):
        _path = str(_path)
        try:f = open(Universals.trEncode(_path, fileSystemEncoding), "w")
        except:f = open(_path, "w")
        f.write(_contents)
        f.close()
        Records.add("Writed", _path)
        
    def writeToBinaryFile(_path, _contents=""):
        _path = str(_path)
        try:f = open(Universals.trEncode(_path, fileSystemEncoding), "wb")
        except:f = open(_path, "w")
        f.write(_contents)
        f.flush()
        f.close()
        Records.add("Writed", _path)
    
    def addToFile(_path, _contents=""):
        _path = str(_path)
        try:f = open(Universals.trEncode(_path, fileSystemEncoding), "a")
        except:f = open(_path, "w")
        f.write(_contents)
        f.close()
        Records.add("Added", _path)
    
    def readTextFile(_path, _contentEncoding = fileSystemEncoding):
        fileDetails = {}
        fileDetails["path"] = _path
        fileDetails["content"] = readFromFile(_path, _contentEncoding)
        #return [getDirName(_path), getBaseName(_path), readFromFile(_path)]  
        return fileDetails
        
    def writeTextFile(_oldFileValues, _newFileValues, _charSet="utf-8"):
        if _oldFileValues["content"]!=_newFileValues["content"] or _charSet!="utf-8":
            writeToFile(_oldFileValues["path"], Universals.trEncode(_newFileValues["content"], _charSet))
        if InputOutputs.getRealPath(_oldFileValues["path"]) != InputOutputs.getRealPath(_newFileValues["path"]):
            return moveOrChange(_oldFileValues["path"], _newFileValues["path"])
        return _oldFileValues["path"]
                
    def clearEmptyDirectories(_path, _isAutoCleanSubFolder=True, _isClear=False):
        #If directory deleted : returned True
        #If directory cleaned : returned False
        if Universals.getBoolValue("isActiveClearGeneral") or _isClear:
            clearUnneededs(_path)
            dontRemovingFilesCount = 0
            filesAndDirectories = readDirectoryAll(_path)
            for nameNo, name in enumerate(filesAndDirectories):
                if isFile(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    if Universals.getBoolValue("isDeleteEmptyDirectories"):
                        for f in Universals.getListFromStrint(Universals.MySettings["ignoredFiles"]):
                            try:
                                if str(f)==name:
                                    dontRemovingFilesCount-=1
                                    break
                            except:pass
                        for ext in Universals.getListFromStrint(Universals.MySettings["ignoredFileExtensions"]):
                            try:
                                if checkExtension(name, ext):
                                    dontRemovingFilesCount-=1
                                    break
                            except:pass
                if isDir(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    if _isAutoCleanSubFolder==False:
                        break
                    if Universals.getBoolValue("isDeleteEmptyDirectories"):
                        for f in Universals.getListFromStrint(Universals.MySettings["ignoredDirectories"]):
                            try:
                                if str(f)==name:
                                    dontRemovingFilesCount-=1
                                    break
                            except:pass
                    if clearEmptyDirectories(_path+"/"+name, _isAutoCleanSubFolder, _isClear):
                        dontRemovingFilesCount-=1
            if dontRemovingFilesCount==0 and Universals.getBoolValue("isDeleteEmptyDirectories"):
                clearIgnoreds(_path)
                removeDir(_path)
                return True
        return False
        
    def clearUnneededs(_path):
        if checkSource(_path, "directory"):
            for f in Universals.getListFromStrint(Universals.MySettings["unneededFiles"]):
                try:
                    if isFile(_path+"/"+str(f)):
                        removeFile(_path+"/"+str(f))
                except:pass
            for f in Universals.getListFromStrint(Universals.MySettings["unneededDirectoriesIfIsEmpty"]):
                try:
                    if isDirEmpty(_path+"/"+str(f)) and f.strip()!="":
                        removeDir(_path+"/"+str(f))
                except:pass
            for f in Universals.getListFromStrint(Universals.MySettings["unneededDirectories"]):
                try:
                    if isDir(_path+"/"+str(f)) and f.strip()!="":
                        removeFileOrDir(_path+"/"+str(f), True)
                except:pass
            for name in readDirectoryAll(_path):
                if isFile(_path+"/"+name):
                    for ext in Universals.getListFromStrint(Universals.MySettings["unneededFileExtensions"]):
                        try:
                            if checkExtension(name, ext):
                                removeFile(_path+"/"+name)
                        except:pass
                
    def clearIgnoreds(_path):
        if checkSource(_path, "directory"):
            for f in Universals.getListFromStrint(Universals.MySettings["ignoredFiles"]):
                try:
                    if isFile(_path+"/"+str(f)):
                        removeFile(_path+"/"+str(f))
                except:pass
            for f in Universals.getListFromStrint(Universals.MySettings["ignoredDirectories"]):
                try:
                    if isDir(_path+"/"+str(f)) and f.strip()!="":
                        removeFileOrDir(_path+"/"+str(f), True)
                except:pass
            for name in readDirectoryAll(_path):
                if isFile(_path+"/"+name):
                    for ext in Universals.getListFromStrint(Universals.MySettings["ignoredFileExtensions"]):
                        try:
                            if checkExtension(name, ext):
                                removeFile(_path+"/"+name)
                        except:pass
    
    def removeFileOrDir(_path, _isDir=False):
        if isWritableFileOrDir(getDirName(_path)):
            if _isDir==False:
                removeFile(_path)
            else:
                for f in readDirectoryAll(_path):
                    if isFile(_path+"/"+f):
                        removeFileOrDir(_path+"/"+f)
                    elif isDir(_path+"/"+f):
                        removeFileOrDir(_path+"/"+f, True)
                removeDir(_path)
    
    def removeOnlySubFiles(_path):
        if isWritableFileOrDir(_path):
            for f in readDirectoryAll(_path):
                if isFile(_path+"/"+f):
                    removeFile(_path+"/"+f)
                elif isDir(_path+"/"+f):
                    removeOnlySubFiles(_path+"/"+f)
    
    def moveOrChange(_oldPath, _newPath, _objectType="file", _actionType="auto", _isQuiet=False):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        isChange=False
        if checkSource(_oldPath, _objectType):
            isChange=True
            _newPath = checkDestination(_oldPath, _newPath, _isQuiet)
        if isChange==True and _newPath:
            if _objectType=="directory" and _actionType=="auto":
                if Universals.getBoolValue("isClearEmptyDirectoriesWhenMoveOrChange"):
                    if clearEmptyDirectories(_oldPath, True, True, Universals.getBoolValue("isAutoCleanSubFolderWhenMoveOrChange")):
                        return _oldPath
            for tDir in appendingDirectories:
                if _newPath==tDir:
                    for name in readDirectoryAll(_oldPath):
                        moveOrChange(_oldPath+"/"+name, _newPath+"/"+name, getObjectType(_oldPath+"/"+name), _actionType, _isQuiet)
                    isChange = False
            if isChange==True:
                moveFileOrDir(_oldPath,_newPath)
            if _objectType=="directory" and _actionType=="auto":
                if Universals.getBoolValue("isClearEmptyDirectoriesWhenMoveOrChange"):
                    if clearEmptyDirectories(_newPath, True, True, Universals.getBoolValue("isAutoCleanSubFolderWhenMoveOrChange")):
                        return _newPath
            if isDir(_newPath)==True and _actionType=="auto":
                if Universals.getBoolValue("isActiveAutoMakeIconToDirectory") and Universals.getBoolValue("isAutoMakeIconToDirectoryWhenMoveOrChange"):
                    checkIcon(_newPath)
            elif _actionType=="auto":
                if Universals.getBoolValue("isActiveAutoMakeIconToDirectory") and Universals.getBoolValue("isAutoMakeIconToDirectoryWhenFileMove"):
                    if isDir(getDirName(_oldPath)):
                        checkIcon(getDirName(_oldPath))
                    if isDir(getDirName(_newPath)):
                        checkIcon(getDirName(_newPath))
            return _newPath
        else:
            return _oldPath
            
    def copyOrChange(_oldPath,_newPath,_objectType="file", _actionType="auto", _isQuiet=False):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        isChange=False
        if checkSource(_oldPath, _objectType):
            isChange=True
            _newPath = checkDestination(_oldPath, _newPath, _isQuiet)
        if isChange==True and _newPath:
            if _objectType=="directory" and _actionType=="auto":
                if Universals.getBoolValue("isClearEmptyDirectoriesWhenCopyOrChange"):
                    if clearEmptyDirectories(_oldPath, True, True, Universals.getBoolValue("isAutoCleanSubFolderWhenCopyOrChange")):
                        return _oldPath
            for tDir in appendingDirectories:
                if _newPath==tDir:
                    for name in readDirectoryAll(_oldPath):
                        copyOrChange(_oldPath+"/"+name, _newPath+"/"+name, getObjectType(_oldPath+"/"+name), _actionType, _isQuiet)
                    isChange = False
            if isChange==True:
                copyFileOrDir(_oldPath,_newPath)
            if isDir(_newPath)==True and _actionType=="auto":
                if Universals.getBoolValue("isActiveAutoMakeIconToDirectory") and Universals.getBoolValue("isAutoMakeIconToDirectoryWhenCopyOrChange"):
                    checkIcon(_newPath)
            return _newPath
        else:
            return _oldPath
    
    def changeDirectories(_values):
        newFilesPath = []
        if len(_values)!=0:
            for no in range(0,len(_values)):
                values = {}
                values["oldPath"] = _values[no][0]
                values["newPath"] = moveOrChange(values["oldPath"], _values[no][1], getObjectType(_values[no][0]))
                newFilesPath.append(values)
                dirPath = getDirName(newFilesPath[-1])
                if Universals.getBoolValue("isClearEmptyDirectoriesWhenFileMove"):
                    clearEmptyDirectories(dirPath, True, True, Universals.getBoolValue("isAutoCleanSubFolderWhenFileMove"))
                if Universals.getBoolValue("isActiveAutoMakeIconToDirectory") and Universals.getBoolValue("isAutoMakeIconToDirectoryWhenFileMove"):
                    checkIcon(dirPath)
        return newFilesPath
        
    def activateSmartCheckIcon():
        global isSmartCheckIcon, willCheckIconDirectories
        isSmartCheckIcon = True
        willCheckIconDirectories = []
    
    def completeSmartCheckIcon():
        global isSmartCheckIcon, willCheckIconDirectories
        isSmartCheckIcon = False
        for iconDir in willCheckIconDirectories:
            checkIcon(iconDir)
        willCheckIconDirectories = []
    
    def checkIcon(_path, _isClear=False):
        global isSmartCheckIcon, willCheckIconDirectories
        if isSmartCheckIcon and _isClear==False:
            if willCheckIconDirectories.count(_path)==0:
                willCheckIconDirectories.append(_path)
        else:
            if _isClear==False:
                coverPath = ""
                coverName = getFirstImageInDirectory(_path)
                if coverName!=None:
                    coverPath = _path + "/" + coverName
                return setIconToDirectory(_path, coverPath)
            elif _isClear:
                return setIconToDirectory(_path)
    
    def getFirstImageInDirectory(_path, _coverNameIfExist=None, _isCheckDelete=False):
        _path = str(_path)
        cover = None
        imageFiles = []
        for fileName in readDirectoryAll(_path):
            if isFile(_path + "/" + fileName):
                if str(fileName.split(".")[0]).lower()==str(_coverNameIfExist).lower():
                    cover = fileName
                if Universals.getListFromStrint(Universals.MySettings["imageExtensions"]).count((fileName.split(".")[-1]).lower()) != 0:
                    imageFiles.append(fileName)
                    if cover == None:
                        for coverName in Universals.getListFromStrint(Universals.MySettings["priorityIconNames"]):
                            if str(fileName.split(".")[0]).lower()==str(coverName).lower():
                                cover = fileName
                                break
        if cover == None and len(imageFiles)>0:
            for imgFile in imageFiles:
                cover = imgFile
                break
        if _isCheckDelete and cover!=None:
            if isWritableFileOrDir(_path):
                if eval(Universals.MySettings["isDeleteOtherImages"].title())==True: 
                    for imgFile in imageFiles:
                        if cover != imgFile:
                            removeFile(_path + "/" + imgFile)
        return cover
        
    def setIconToDirectory(_path, _iconName=""):
        _path = str(_path)
        if isDir(_path):
            if _iconName==None:
                return False
            _iconName = str(_iconName).strip()
            returnValue, isChanging, isChange, isCorrectFileContent, rows = False, False, True, False, []
            if isFile(_iconName):
                if str(_path)==str(getDirName(_iconName)):
                    _iconName = "./" + getBaseName(_iconName)
                try:
                    info = readFromFile(_path + "/.directory")
                    if info.find("[Desktop Entry]")==-1:
                        info = "[Desktop Entry]\n" + info
                    isCorrectFileContent = True
                except:
                    info = "[Desktop Entry]"
                rows = info.split("\n")
                for rowNo in range(len(rows)):
                    if rows[rowNo][:5] == "Icon=":
                        if len(rows[rowNo])>5:
                            isFileExist = False
                            if rows[rowNo][5]=="." and isFile(_path + str(rows[rowNo][6:])):
                                isFileExist=True
                            elif rows[rowNo][5]!="." and isFile(rows[rowNo][5:]):
                                isFileExist=True
                            if isFileExist:
                                if Universals.getBoolValue("isChangeExistIcon")==False:
                                    isChange = False
                        isChanging = True
                        rows[rowNo] = "Icon=" + _iconName 
                        returnValue = True
                if isChange:
                    if isChanging==False:
                        rows.append("Icon=" + _iconName)
                        returnValue = True
                if isCorrectFileContent:
                    rowNoStrDesktopEntry = -1
                    rowNoStrIcon = -1
                    for rowNo in range(len(rows)):
                        if rows[rowNo].find("[Desktop Entry]")!=-1:
                            rowNoStrDesktopEntry = rowNo
                        elif rows[rowNo].find("Icon=")!=-1:
                            rowNoStrIcon = rowNo
                    if rowNoStrDesktopEntry != rowNoStrIcon - 1:
                        rows[rowNoStrDesktopEntry] += "\n" + rows[rowNoStrIcon]
                        rows[rowNoStrIcon] = ""
            else:
                if isFile(_path + "/.directory"):
                    info = readFromFile(_path + "/.directory")
                    rows = info.split("\n")
                    for rowNo in range(len(rows)):
                        if len(rows[rowNo])>4:
                            if rows[rowNo][:5] == "Icon=":
                                rows[rowNo] = ""
                                break
            info=""
            for row in rows:
                if row.strip()!="":
                    info+=row+"\n"
            writeToFile(_path + "/.directory", info)
            return returnValue
        else:
            return False
        
    def getIconFromDirectory(_path):
        iconPath, isCorrectedFileContent = None, True
        if isFile(_path + "/.directory"):
            info = readFromFile(_path + "/.directory")
            if info.find("[Desktop Entry]")==-1 and len(info)>0:
                isCorrectedFileContent = False
            if info.find("[Desktop Entry]") > info.find("Icon=") and info.find("Icon=")>-1:
                isCorrectedFileContent = False
            rows = info.split("\n")
            for rowNo in range(len(rows)):
                if rows[rowNo][:5] == "Icon=":
                    if len(rows[rowNo])>5:
                        if rows[rowNo][5]=="." and isFile(_path + str(rows[rowNo][6:])):
                            iconPath = _path + str(rows[rowNo][6:])
                        elif rows[rowNo][5]!="." and isFile(rows[rowNo][5:]):
                            iconPath = rows[rowNo][5:]
                        elif rows[rowNo][5]==".":
                            iconPath = _path + str(rows[rowNo][6:])
                            isCorrectedFileContent = False
                        else:
                            iconPath = rows[rowNo][5:]
                            isCorrectedFileContent = False
        return iconPath, isCorrectedFileContent

    def clearPackagingDirectory(_path):
        if checkSource(_path, "directory"):
            _path = str(_path)
            if Universals.getBoolValue("isClearEmptyDirectoriesWhenPath"):
                clearEmptyDirectories(_path, Universals.getBoolValue("isAutoCleanSubFolderWhenPath"))
            for f in Universals.getListFromStrint(Universals.MySettings["packagerUnneededFiles"]):
                if isFile(_path+"/"+f):
                    removeFile(_path+"/"+f)
            for d in Universals.getListFromStrint(Universals.MySettings["packagerUnneededDirectories"]):
                if isExist(_path+"/"+d):
                    removeFileOrDir(_path+"/"+d, True)
            dontRemovingFilesCount = 0
            filesAndDirectories = readDirectoryAll(_path)
            for nameNo, name in enumerate(filesAndDirectories):
                if isFile(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    for ext in Universals.getListFromStrint(Universals.MySettings["packagerUnneededFileExtensions"]):
                        try:
                            if checkExtension(name, ext):
                                removeFile(_path+"/"+name)
                                dontRemovingFilesCount-=1
                                continue
                        except:pass
                    try:
                        if name[-1:]=="~":
                            removeFile(_path+"/"+name)
                            dontRemovingFilesCount-=1
                            continue
                    except:pass
                if isDir(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    if clearPackagingDirectory(_path+"/"+name)==False:
                        dontRemovingFilesCount-=1
            if dontRemovingFilesCount==0 and Universals.getBoolValue("isPackagerDeleteEmptyDirectories"):
                removeDir(_path)
                return False
            return True
        else:
            False
            
    def clearCleaningDirectory(_path):
        if checkSource(_path, "directory"):
            _path = str(_path)
            if Universals.getBoolValue("isClearEmptyDirectoriesWhenClear"):
                clearEmptyDirectories(_path, Universals.getBoolValue("isAutoCleanSubFolderWhenClear"))
            for f in Universals.getListFromStrint(Universals.MySettings["cleanerUnneededFiles"]):
                if isFile(_path+"/"+f):
                    removeFile(_path+"/"+f)
            for d in Universals.getListFromStrint(Universals.MySettings["cleanerUnneededDirectories"]):
                if isExist(_path+"/"+d):
                    removeFileOrDir(_path+"/"+d, True)
            dontRemovingFilesCount = 0
            filesAndDirectories = readDirectoryAll(_path)
            for nameNo, name in enumerate(filesAndDirectories):
                if isFile(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    for ext in Universals.getListFromStrint(Universals.MySettings["cleanerUnneededFileExtensions"]):
                        try:
                            if checkExtension(name, ext):
                                removeFile(_path+"/"+name)
                                dontRemovingFilesCount-=1
                                continue
                        except:pass
                    try:
                        if name[-1:]=="~":
                            removeFile(_path+"/"+name)
                            dontRemovingFilesCount-=1
                            continue
                    except:pass
                if isDir(_path+"/"+name):
                    dontRemovingFilesCount+=1
                    if clearPackagingDirectory(_path+"/"+name)==False:
                        dontRemovingFilesCount-=1
            if dontRemovingFilesCount==0 and Universals.getBoolValue("isCleanerDeleteEmptyDirectories"):
                removeDir(_path)
                return False
            return True
        else:
            False

    def makePack(_filePath, _packageType, _sourcePath):
        import tarfile
        _filePath, _sourcePath = str(_filePath), str(_sourcePath)
        if isDir(_filePath):
            return False
        try:tar = tarfile.open(Universals.trEncode(_filePath, fileSystemEncoding), "w:" + _packageType)
        except:tar = tarfile.open(_filePath, "w:" + _packageType)
        try:tar.add(Universals.trEncode(_sourcePath, fileSystemEncoding), "")
        except:tar.add(_sourcePath, "")
        tar.close()
        Records.add("Packed", _filePath)
        return True
        
    def extractPack(_oldPath, _newPath):
        _oldPath, _newPath = str(_oldPath), str(_newPath)
        import tarfile
        while 1==1:
            try:
                try:tar = tarfile.open(Universals.trEncode(_oldPath, fileSystemEncoding), "r:gz")
                except:tar = tarfile.open(_oldPath, "r:gz")
                break
            except:
                time.sleep(1)
        try:tar.extractall(Universals.trEncode(_newPath, fileSystemEncoding), members=tar.getmembers())
        except:tar.extractall(_newPath, members=tar.getmembers())
        tar.close()
        Records.add("Extracted", _oldPath, _newPath)
        
    def clearTempFiles():
        import tempfile
        for fileName in readDirectoryAll(tempfile.gettempdir()):
            if fileName[:15] == "HamsiManager":
                if isDir(tempfile.gettempdir()+"/"+fileName):
                    removeFileOrDir(tempfile.gettempdir()+"/"+fileName, True)
                else:
                    removeFileOrDir(tempfile.gettempdir()+"/"+fileName)
                    
    def getFileTree(_path, _subDirectoryDeep=-1, _formatType="html", _extInfo="no"):
        from Universals import translate
        _path = str(_path)
        files = readDirectoryWithSubDirectories(_path, _subDirectoryDeep, True)
        info = ""
        if _formatType=="html":
            if _extInfo=="no":
                pass
            elif _extInfo=="title":
                info += " \n <h3>%s </h3> \n" % (str(translate("Tables", "File Tree")))
                info += " %s<br> \n" % (_path)
            dirNumber = _path.count("/")
            findStrings, replaceStrings = [], []
            for x, file in enumerate(files):
                if isDir(file):
                    findStrings.append(file)
                    replaceStrings.append((Universals.getUtf8Data("upright") + "&nbsp;&nbsp;&nbsp;"*(file.count("/")-dirNumber)) + Universals.getUtf8Data("upright+right") + "&nbsp;")
            findStrings.reverse()
            replaceStrings.reverse()
            fileList = list(range(len(files)))
            for x, file in enumerate(files):
                fileList[x] = file + "<br> \n"
                for  y, fstr in enumerate(findStrings):
                    if file!=fstr:
                        fileList[x] = fileList[x].replace(fstr + "/", replaceStrings[y])
                if x>0:
                    tin = fileList[x-1].find(Universals.getUtf8Data("upright+right"))
                    tin2 = fileList[x].find(Universals.getUtf8Data("upright+right"))
                    if tin>tin2:
                        fileList[x-1] = fileList[x-1].replace(Universals.getUtf8Data("upright+right"), Universals.getUtf8Data("up+right"))
            for x, fileName in enumerate(fileList):
                if x!=len(fileList)-1:
                    info += fileName.replace(_path + "/", Universals.getUtf8Data("upright+right") + "&nbsp;")
                else:
                    info += fileName.replace(_path + "/", Universals.getUtf8Data("up+right") + "&nbsp;")
        elif _formatType=="plainText":
            if _extInfo=="no":
                pass
            elif _extInfo=="title":
                info += " %s \n" % (str(translate("Tables", "File Tree")))
                info += _path + "\n"
            dirNumber = _path.count("/")
            findStrings, replaceStrings = [], []
            for x, file in enumerate(files):
                if isDir(file):
                    findStrings.append(file)
                    replaceStrings.append((Universals.getUtf8Data("upright") + "   "*(file.count("/")-dirNumber)) + Universals.getUtf8Data("upright+right") + " ")
            findStrings.reverse()
            replaceStrings.reverse()
            fileList = list(range(len(files)))
            for x, file in enumerate(files):
                fileList[x] = file + "\n"
                for  y, fstr in enumerate(findStrings):
                    if file!=fstr:
                        fileList[x] = fileList[x].replace(fstr + "/", replaceStrings[y])
                if x>0:
                    tin = fileList[x-1].find(Universals.getUtf8Data("upright+right"))
                    tin2 = fileList[x].find(Universals.getUtf8Data("upright+right"))
                    if tin>tin2:
                        fileList[x-1] = fileList[x-1].replace(Universals.getUtf8Data("upright+right"), Universals.getUtf8Data("up+right"))
            for x, fileName in enumerate(fileList):
                if x!=len(fileList)-1:
                    info += fileName.replace(_path + "/", Universals.getUtf8Data("upright+right") + " ")
                else:
                    info += fileName.replace(_path + "/", Universals.getUtf8Data("up+right") + " ")
        return info
            
    def fixToSize(_path, _size, _clearFrom="head"):
        if isFile(_path):
            while getSize(_path) > _size:
                if _clearFrom=="head":
                    try:contents = readFromFile(_path)[500:]
                    except:
                        try:contents = readFromFile(_path)[200:]
                        except:contents = readFromFile(_path)[20:]
                else:
                    try:contents = readFromFile(_path)[:-500]
                    except:
                        try:contents = readFromFile(_path)[:-200]
                        except:contents = readFromFile(_path)[:-20]
                writeToFile(_path, contents)
        
    def getHashDigest(_filePath, _hashType="MD5"):
        try:
            import hashlib
            if _hashType=="MD5":
                m = hashlib.md5()
            elif _hashType=="SHA1":
                m = hashlib.sha1()
            elif _hashType=="SHA224":
                m = hashlib.sha224()
            elif _hashType=="SHA256":
                m = hashlib.sha256()
            elif _hashType=="SHA384":
                m = hashlib.sha384()
            elif _hashType=="SHA512":
                m = hashlib.sha512()
            m.update(readFromBinaryFile(_filePath))
            return m.hexdigest()
        except:
            #for x < python 2.5
            try:
                if _hashType=="MD5":
                    import md5
                    return md5.new(readFromBinaryFile(_filePath)).hexdigest()
                elif _hashType=="SHA1":
                    import sha
                    return sha.new(readFromBinaryFile(_filePath)).hexdigest()
            except:
                return False
        
    def createHashDigestFile(_filePath, _digestFilePath=None, _hashType="MD5", _isAddFileExtension=True, _digestContent=None):
        if _digestContent==None:
            _digestContent = getHashDigest(_filePath, _hashType)
        fileExtension = ""
        if _isAddFileExtension:
            fileExtension = _hashType.lower()
        if _digestFilePath==None:
            _digestFilePath = _filePath
        writeToFile(_digestFilePath + fileExtension, _digestContent)
        return True
        
        try:
            import hashlib
            return ["MD5", "SHA1", "SHA224", "SHA256", "SHA384", "SHA512"]
        except:
            #for x < python 2.5
            hashTypes = []
            try:
                import md5
                hashTypes.append("MD5")
            except:pass
            try:
                import md5
                hashTypes.append("SHA1")
            except:pass
            return hashTypes
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import sys,os

import Variables
from MyObjects import *
import Universals
import Settings
import InputOutputs, Records
import traceback
import logging
from RoutineChecks import isQuickMake, QuickMakeParameters, myArgvs
iSClosingInErrorReporting = False

class ReportBug(MDialog):
    global isClose
    isClose=False
    def __init__(self, _isOnlyReport=False, _hideFixMe=False):
        global errorDetails, isClose
        lastErrorDetails = str(sys.exc_info())
        lastErrorDetailsValues = sys.exc_info()
        MainWindow = Universals.MainWindow
        if Universals.isStartingSuccessfully==True:
            isShowFixMe = False
        else:
            isShowFixMe = True
        try:MDialog.__init__(self, MainWindow)
        except:MDialog.__init__(self, None)
        QtWebKit = getMyObject("QtWebKit")
        self.pathOfReportFile = ""
        self.isOnlyReport=True
        isClose=False
        self.isLoading=True
        errorDetails = "<b>" + str(translate("ReportBug", "Please check your personal information from this table."))+"</b><br>"
        if _isOnlyReport==False:
            Universals.isRaisedAnError = True
            realErrorDetails = str(self.formatExceptionInfo())
            realErrorDetails += "<hr>"
            self.isOnlyReport=False
            realErrorDetails += lastErrorDetails
            cla, error, trbk = lastErrorDetailsValues
            try:
                excArgs = error.__dict__["args"]
            except:
                excArgs = ""
            try:realErrorDetails += "<p><b>" + str(translate("ReportBug", "Error : ")) + "</b>" + traceback.format_tb(trbk, 5) + "</p>"
            except:pass
            realErrorDetails += ("<p><b>" + str(translate("ReportBug", "Error Name : ")) + "</b>" + str(cla.__name__) + "<br><b>" +
                            str(translate("ReportBug", "Error : ")) + "</b>"+str(error)+"<br><b>" +
                            str(translate("ReportBug", "Error arguments : ")) + "</b>"+str(excArgs)+"</p><hr><p><b>" +
                            str(translate("ReportBug", "Last Signal Sender (Object Name,Object Text) : ")) + "</b>\"")
            try:realErrorDetails +=Universals.trUnicode(self.sender().objectName())+"&quot;,&quot;"
            except:realErrorDetails +="&quot;,&quot;"
            try:realErrorDetails +=Universals.trUnicode(self.sender().text())+"&quot;"
            except:realErrorDetails +="&quot;"
            realErrorDetails = realErrorDetails.replace("\\n", "<br>").replace("\'", "&#39;")
        try:Records.saveAllRecords()
        except:pass
        try:Universals.saveSettings()
        except:pass
        try:
            if Universals.isActivePyKDE4==True:
                self.setButtons(MDialog.NoDefault)
        except:pass
        try:
            try:
                errorDetails+="<p><b>"+str(translate("ReportBug", "Contents Directory : "))+"</b>" + Universals.MainWindow.FileManager.getCurrentDirectoryPath() + "</p>"
            except:pass
            errorDetails +="<hr><p><h3>"+str(translate("ReportBug", "Table Contents : "))+"</h3>"
            import Tables
            errorDetails += Tables.exportValues("return", "html", "no")
        except:pass
        try:
            errorDetails +="</p><hr><p><h3>"+str(translate("ReportBug", "File Information : "))+"</h3><table border=1>"
            for rowNo in range(len(Universals.MainWindow.Table.currentTableContentValues)):
                filePath = Universals.MainWindow.Table.currentTableContentValues[rowNo]["path"]
                errorDetails +="<tr><td>" 
                try:errorDetails += str(Universals.trUnicode(filePath, InputOutputs.fileSystemEncoding))
                except:
                    try:errorDetails += str(filePath) 
                    except:errorDetails += filePath
                errorDetails +="</td></tr>"
            errorDetails +="</table></p><hr><p><h3>"+str(translate("ReportBug", "File Details : "))+"</h3><table border=1>"
            for rowNo in range(len(Universals.MainWindow.Table.currentTableContentValues)):
                errorDetails +="<tr>"
                for key, value in Universals.MainWindow.Table.currentTableContentValues[rowNo]:
                    errorDetails +="<td>"
                    try:errorDetails +=str(value)
                    except:errorDetails +=value
                    errorDetails +="</td>"
                errorDetails +="</tr>"
            errorDetails+="</table></p>"
        except:pass
        errorDetails += "<b>" + str(translate("ReportBug", "Active Dialog`s Titles : ")) + "</b>"
        try:errorDetails += str(Universals.HamsiManagerApp.activeModalWidget().windowTitle())+","
        except:errorDetails += "<br>"
        try:errorDetails += str(Universals.HamsiManagerApp.activePopupWidget().windowTitle())+","
        except:errorDetails += "<br>"
        try:errorDetails += str(Universals.HamsiManagerApp.activeWindow().windowTitle())+","
        except:errorDetails += "<br>"
        errorDetails += "<br>"
        try:
            errorDetails += "<b>" + str(translate("ReportBug", "Application Version : ")) + "</b>"
            errorDetails += str(Universals.HamsiManagerApp.applicationVersion())+"<br>"
        except:
            errorDetails += "<br>"
        try:
            errorDetails += "<b>" + str(translate("ReportBug", "Is Starting Successfully : ")) + "</b>"
            errorDetails += str(Universals.isStartingSuccessfully) + "<br>"
            errorDetails += "<b>" + str(translate("ReportBug", "Is Reporting Manuel : ")) + "</b>"
            errorDetails += str(_isOnlyReport) + "<br>"
            errorDetails += "<b>" + str(translate("ReportBug", "Is Quick Make : ")) + "</b>"
            errorDetails += str(isQuickMake) + "<br>"
            errorDetails += "<b>" + str(translate("ReportBug", "Quick Make Parameters : ")) + "</b>"
            errorDetails += str(QuickMakeParameters) + "<br>"
            errorDetails += "<b>" + str(translate("ReportBug", "My Parameters : ")) + "</b>"
            errorDetails += str(myArgvs) + "<br>"
            errorDetails += "<b>FileSystemCharSet : </b>"
            errorDetails += str(Variables.defaultFileSystemEncoding) + " / " + str(sys.getfilesystemencoding().lower()) + "<br>"
            errorDetails += "<b>SystemCharSet : </b>"
            errorDetails += str(sys.getdefaultencoding().lower()) + "<br>"
            try:
                errorDetails += "<b>OS Name : </b>"
                errorDetails += str(os.name) + "<br>"
            except:
                errorDetails += "<br>"
            try:
                import platform
                errorDetails += "<b>Python Version : </b>"
                errorDetails += str(platform.python_version()) + "<br>"
                errorDetails += "<b>uname : </b>"
                errorDetails += str(platform.uname()) + "<br>"
                try:
                    errorDetails += "<b>Linux Distribution : </b>"
                    errorDetails += str(platform.linux_distribution()) + "<br>"
                except:
                    errorDetails += "<br>"
            except:
                errorDetails += "<br>"
            if Variables.MyObjectName == "PyQt4":
                try:
                    errorDetails += "<b>PyQt4 (Qt) Version : </b>"
                    errorDetails += str(PYQT_VERSION_STR) + " ("+ str(MT_VERSION_STR) + ")<br>"
                except:
                    errorDetails += "<br>"
                try:
                    from PySide import QtCore
                    errorDetails += "<b>PySide (Qt) Version : </b>"
                    errorDetails += str(QtCore.qVersion()) + " (" + QtCore.QT_VERSION_STR +")<br>"
                except:
                    errorDetails += "<br>"
            elif Variables.MyObjectName == "PySide":
                try:
                    errorDetails += "<b>PySide (Qt) Version : </b>"
                    errorDetails += str(PYQT_VERSION_STR) + " ("+ str(MT_VERSION_STR) + ")<br>"
                except:
                    errorDetails += "<br>"
                try:
                    from PyQt4 import QtCore
                    errorDetails += "<b>PyQt4 (Qt) Version : </b>"
                    errorDetails += str(QtCore.qVersion()) + " (" + QtCore.QT_VERSION_STR +")<br>"
                except:
                    errorDetails += "<br>"
            try:
                from PyKDE4 import kdecore
                errorDetails += "<b>PyKDE4 Version : </b>"
                errorDetails += str(kdecore.versionString()) + "<br>"
            except:
                errorDetails += "<br>"
            for keyName in Universals.MySettings:
                if Variables.willNotReportSettings.count(keyName)==0:
                    errorDetails += "<b>" + str(keyName) + " : " + "</b>"
                    errorDetails += str(Universals.MySettings[keyName]) + "<br>"
        except:pass
        try:
            import Tables
            errorDetails += "<b>" + str(translate("ReportBug", "Table Type No : ")) + "</b>" + str(Universals.tableType) +"<br>"
        except:pass
        if _isOnlyReport==False:
            errorDetails += str(realErrorDetails)
        try:
            import Dialogs
            if Dialogs.pnlState!=None:
                Dialogs.showState("", 1, 1)
        except:pass
        pnlMain = MWidget(self)
        self.vblMain = MVBoxLayout(pnlMain)
        self.pbtnClose = MPushButton(translate("ReportBug", "Close (Please Report This Bug First.)"))
        self.pbtnShowDetailsPage = MPushButton(translate("ReportBug", "Show Details File"))
        self.pbtnCheckUpdate = MPushButton(translate("ReportBug", "Check Update"))
        self.teErrorDetails = MTextEdit()  
        self.wvWeb = QtWebKit.QWebView()
        self.createErrorPage(errorDetails)
        self.connect(self.wvWeb,SIGNAL("loadProgress(int)"),self.loading)
        try:
            self.teErrorDetails.setHtml(trForUI(errorDetails.replace("<hr>", "")))
        except:
            self.teErrorDetails.setHtml(translate("ReportBug", "I cannot send the error details due to some character errors.<br>To see the details, please click on the \"Show details file\" button."))
            self.teErrorDetails.setEnabled(False)
        self.connect(self.teErrorDetails,SIGNAL("textChanged()"), self.errorDetailsChanged)
        self.connect(self.pbtnClose,SIGNAL("clicked()"), self.close)
        self.connect(self.pbtnShowDetailsPage,SIGNAL("clicked()"), self.showDetailsPage)
        self.connect(self.pbtnCheckUpdate,SIGNAL("clicked()"), self.checkUpdate)
        self.teErrorDetails.setMinimumHeight(220)
        self.vblMain.addWidget(self.teErrorDetails, 10) 
        self.vblMain.addWidget(self.wvWeb, 10) 
        hbox0 = MHBoxLayout()
        hbox0.addWidget(self.pbtnShowDetailsPage,1)
        hbox0.addStretch(2)
        hbox0.addWidget(self.pbtnCheckUpdate,1)
        hbox0.addWidget(self.pbtnClose,1)
        self.vblMain.addLayout(hbox0, 1)
        try:
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(self.vblMain)
        except:
            self.setLayout(self.vblMain)
        self.setWindowTitle(translate("ReportBug", "Please Report This Bug!.."))
        self.setMaximumSize(600, 375)  
        self.wvWeb.setMinimumHeight(272)
        self.show()
        self.setMaximumSize(10000, 10000)
        if isShowFixMe == True and isQuickMake==False and _hideFixMe==False and Universals.loggingLevel!=logging.DEBUG:
            try:
                import Dialogs
                answer = Dialogs.askSpecial(translate("ReportBug", "I Have A Suggestion!"),
                            translate("ReportBug", "<b>Please check the following: ;</b><br>"+
                            "<b>1-)</b>If you have received this error when you were checking the last folder, reset the \"Last Directory\",<br>"+
                            "<b>2-)</b>If you have received this error due to your changed settings, reset the \"Settings\",<br>"+
                            "<b>3-)</b>If you continue to receive this error even after resetting the settings, reset \"All\".<br>"+
                            "<br><b>You can enable Hamsi Manager to run as normal.<br>Please take a moment to send us the error report.</b>"), 
                            translate("ReportBug", "Last Directory"), 
                            translate("ReportBug", "All"), 
                            translate("ReportBug", "Settings"), 
                            translate("ReportBug", "Ignore"))
                if answer==translate("ReportBug", "Last Directory"):
                    Settings.setting().setValue("lastDirectory", Universals.trQVariant(trForM(Variables.userDirectoryPath)))
                elif answer==translate("ReportBug", "Settings"):
                    Settings.reFillSettings(True)
                elif answer==translate("ReportBug", "All"):
                    Settings.reFillAll(True)
            except:pass
    
    def formatExceptionInfo(self, maxTBlevel=5):
        cla, exc, trbk = sys.exc_info()
        excName = cla.__name__
        try:
            excArgs = exc.__dict__["args"]
        except:
            excArgs = "<no args>"
        excTb = traceback.format_tb(trbk, maxTBlevel)
        return (excName, excArgs, excTb)
    
    def createErrorPage(self, _errorDetails, _userNote="", _userName="", _mail=""):
        _errorDetails = _errorDetails.replace("\"", "&quot;").replace("\'", "&#39;")
        self.isLoading=False
        language = "en_GB"
        if "language" in Universals.MySettings:
            language = Universals.MySettings["language"]
        htmlString=('<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'+
                    '<html xmlns="http://www.w3.org/1999/xhtml"><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8" /><title>Hamsi Manager</title></head><body>'+
                    '<center>'+
                    '<form action="http://hamsiapps.com/ForMyProjects/ReportBug.php" method="post">'+
                    '<TABLE><TR><TD valign="top">%s'
                    '</TD><TD colspan=2><textarea ROWS="7" COLS="40" name="userNotes">%s</textarea></TD></TR></TABLE>'+
                    '<TABLE><TR><TD valign="top" colspan=2>%s</TD><TD align="right"><input type="search" name="nameAndSurname" value="%s"></input></TD></TR>'+
                    '<TR><TD valign="top" colspan=2>%s</TD><TD align="right"><input type="search" name="mail" value="%s"></input></TD></TR></TABLE>'+
                    '<TABLE><TR><TD align="right"><input name="send" type="submit" value="&nbsp;&nbsp;&nbsp;%s&nbsp;&nbsp;&nbsp;&nbsp;"></TD></TR></TABLE>'+
                    '<INPUT TYPE="hidden" name="error" value="~ERRORDETAILS~" />'+
                    '<INPUT TYPE="hidden" name="thankYouMessages" value="%s" />'+
                    '<INPUT TYPE="hidden" name="p" value="HamsiManager" />'+
                    '<INPUT TYPE="hidden" name="l" value="' + str(language) + '" />'+
                    '<INPUT TYPE="hidden" name="v" value="' + str(Variables.intversion) + '" /></form>'+
                    '~ADDITIONALDETAILS~</center></body></html>'
                    ) % (
                    str(translate("ReportBug", "<b>Error description :</b> <br>(Be can null)<br><b>Note:</b>Please write what you did before you received the error here.")), 
                    _userNote, 
                    str(translate("ReportBug", "<b>Name and Surname :</b> (Be can null)")), 
                    _userName, 
                    str(translate("ReportBug", "<b>E-mail address :</b> (Be can null)<br><b>Note:</b>Will be kept strictly confidential. It will be used solely to report you back once the problem is solved..")), 
                    _mail, 
                    str(translate("ReportBug", "Report Bug")), 
                    str(translate("ReportBug", "Thank you for sending us your error report. You have already contributed a lot to make the next release even better..<br>")))
        self.createErrorFile(htmlString.replace("~ERRORDETAILS~", _errorDetails).replace("~ADDITIONALDETAILS~", _errorDetails))
        encodedType = ""
        try:
            errorDetails = Universals.trDecode(_errorDetails, "utf-8")
            encodedType = "utf-8"
        except:
            try:
                errorDetails = str(Universals.trUnicode(_errorDetails, "iso-8859-9"))
                t = Universals.trDecode(errorDetails, "utf-8")
                encodedType = "iso-8859-9"
            except:
                try:
                    errorDetails = str(Universals.trUnicode(_errorDetails, "cp-1254"))
                    t = Universals.trDecode(errorDetails, "utf-8")
                    encodedType = "cp-1254"
                except:
                    for charName in Variables.getCharSets():
                        try:
                            errorDetails = str(Universals.trUnicode(_errorDetails, charName))
                            t = Universals.trDecode(errorDetails, "utf-8")
                            encodedType = charName
                        except:pass
                    if encodedType=="":
                        errorDetails = ""
        htmlString = htmlString.replace("~ERRORDETAILS~", errorDetails).replace("~ADDITIONALDETAILS~", str(translate("ReportBug", "<b>(Is Encoded With %s.)</b>")) % (encodedType))
        try:self.wvWeb.setHtml(trForUI(htmlString))
        except:
            self.teErrorDetails.setVisible(False)   
            self.wvWeb.setUrl(MUrl(trForM(self.pathOfReportFile)))
        self.isLoading=True
    
    def errorDetailsChanged(self):
        self.createErrorPage(str(self.teErrorDetails.toHtml()))
        pass
    
    def loading(self, _value):
        if self.isLoading:
            if (_value==100):
                self.pbtnClose.setText(translate("ReportBug", "Close"))

    def closeEvent(self, _event):
        global isClose, iSClosingInErrorReporting
        isClose=True
        try:
            self.close()
            if self.isOnlyReport==False and Universals.loggingLevel!=logging.DEBUG:
                iSClosingInErrorReporting = True
                self.parent().close()
        except:pass
        
    def createErrorFile(self, _errorDetails):
        if self.pathOfReportFile=="":
            import tempfile, random
            self.pathOfReportFile = tempfile.gettempdir() + "/HamsiManager-ErrorOutput-"+ str(random.randrange(0, 1000000))+".html"
            InputOutputs.writeToFile(self.pathOfReportFile, _errorDetails)
    
    def showDetailsPage(self):
        self.teErrorDetails.setVisible(False)   
        self.wvWeb.setUrl(MUrl(trForM(self.pathOfReportFile)))
        
    def checkUpdate(self):
        import UpdateControl
        UpdateControl.UpdateControl(self)
        
        
        



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import time
import Universals
import InputOutputs
import Settings
import logging
from Universals import translate

class Records():
    global add, create, read, setTitle, showInWindow, clearRecords, recordContents, isSetedTitle, saveAllRecords,recordContents, checkSize, recordType, lastRecordType, setRecordType, restoreRecordType
    recordContents = ""
    isSetedTitle = False
    recordType = 0
    lastRecordType = 0
    #recordType : 0=Normal, 1=Debug
    
    def create():
        global recordContents
        recordContents += str(translate("Records", "Hamsi Manager Record File - Time Created : ")) + str(time.strftime("%d.%m.%Y %H:%M:%S"))+"\n"
    
    def setTitle(_title):
        global isSetedTitle, recordContents
        if "isSaveActions" not in Universals.MySettings.keys() or Universals.getBoolValue("isSaveActions"):
            recordContents += str(_title) + "\n"
        if Universals.loggingLevel==logging.DEBUG:
            print (_title)
        isSetedTitle = True
    
    def add(_action, _previous="", _now=""):
        global recordContents
        if "isSaveActions" not in Universals.MySettings.keys() or Universals.getBoolValue("isSaveActions"):
            if recordType==0 or (recordType==1 and Universals.loggingLevel==logging.DEBUG):
                recordContents += str(_action + " ::::::: '") + str(_previous) + "' >>>>>>>> '" + str(_now) + "' <<<<<<< " + str(time.strftime("%d.%m.%Y %H:%M:%S"))+"\n"
        if Universals.loggingLevel==logging.DEBUG:
            print (str(_action + " ::::::: '") + str(_previous) + "' >>>>>>>> '" + str(_now) + "' <<<<<<< " + str(time.strftime("%d.%m.%Y %H:%M:%S"))+"\n")
        
    def setRecordType(_recordType):
        global lastRecordType, recordType
        lastRecordType = recordType
        recordType = _recordType
        
    def restoreRecordType():
        global recordType
        recordType = lastRecordType
        
    def saveAllRecords():
        global recordContents, isSetedTitle
        if "isSaveActions" not in Universals.MySettings.keys() or Universals.getBoolValue("isSaveActions"):
            if InputOutputs.isFile(Universals.recordFilePath)==False:
                create()
            setRecordType(1)
            InputOutputs.addToFile(Universals.recordFilePath, recordContents)
            restoreRecordType()
        recordContents = ""
        isSetedTitle = False
    
    def checkSize():
        setRecordType(1)
        InputOutputs.fixToSize(Universals.recordFilePath, (int(Universals.MySettings["maxRecordFileSize"])*1024))
        restoreRecordType()
        
    def read(_isShowErrorDialog=True):
        if InputOutputs.isFile(Universals.recordFilePath)==True:
            return InputOutputs.readFromFile(Universals.recordFilePath)
        else:
            if _isShowErrorDialog:
                import Dialogs
                Dialogs.showError(translate("Records", "Cannot Find The Record File"), 
                            translate("Records", "Record file not found."))
            return False
            
    def clearRecords():
        InputOutputs.writeToFile(Universals.recordFilePath, str(translate("Records", "Hamsi Manager Record File - Time Clear : ")) + str(time.strftime("%d.%m.%Y %H:%M:%S"))+"\n")
        try:dialog.close()
        except:pass
        
    def showInWindow():
        from MyObjects import MDialog, MWidget, MVBoxLayout, MHBoxLayout, MTextEdit, MTextOption, MPushButton, SIGNAL, MObject, trForUI
        import Organizer
        global dialog
        recordString = read()
        if recordString != False:
            dialog = MDialog(Universals.MainWindow)
            if Universals.isActivePyKDE4==True:
                dialog.setButtons(MDialog.NoDefault)
            dialog.setWindowTitle(translate("Records", "Last Records"))
            pnlMain = MWidget(dialog)
            vblMain = MVBoxLayout(pnlMain)
            info = MTextEdit()
            info.setPlainText(trForUI(recordString))
            info.setWordWrapMode(MTextOption.ManualWrap)
            pbtnClose = MPushButton(translate("Records", "OK"))
            pbtnClear = MPushButton(translate("Records", "Clear"))
            MObject.connect(pbtnClose, SIGNAL("clicked()"), dialog.close)
            MObject.connect(pbtnClear, SIGNAL("clicked()"), clearRecords)
            vblMain.addWidget(info)
            hblBox = MHBoxLayout()
            hblBox.addWidget(pbtnClear)
            hblBox.addWidget(pbtnClose)
            vblMain.addLayout(hblBox)
            if Universals.isActivePyKDE4==True:
                dialog.setMainWidget(pnlMain)
            else:
                dialog.setLayout(vblMain)
            dialog.setMinimumSize(550, 400)
            dialog.show()
            
            
            



## This file is part of HamsiManager.
## 
## Copyright (c) 2010 Murat Demir <mopened@gmail.com>      
##
## Hamsi Manager is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## Hamsi Manager is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with HamsiManager; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


from MyObjects import *
import Universals
import Dialogs
import InputOutputs
import Options
import Organizer

MyDialog, MyDialogType, MyParent = getMyDialog()

class Cleaner(MyDialog):
    def __init__(self, _directory):
        MyDialog.__init__(self, MyParent)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setButtons(MyDialog.NoDefault)
        elif MyDialogType=="MMainWindow":
            self.setObjectName("Cleaner")
            Universals.MainWindow = self
        newOrChangedKeys = Universals.newSettingsKeys + Universals.changedDefaultValuesKeys
        wOptionsPanel = Options.Options(None, "clear", None, newOrChangedKeys)
        lblPleaseSelect = MLabel(translate("Cleaner", "Directory"))
        self.pbtnClear = MPushButton(translate("Cleaner", "Clear"))
        self.pbtnClose = MPushButton(translate("Cleaner", "Close"))
        self.lePathOfProject = MLineEdit(trForM(_directory))
        self.pbtnClear.setToolTip(translate("Cleaner", "Directory you selected will is cleared"))
        self.pbtnSelectProjectPath = MPushButton(translate("Cleaner", "Browse"))
        self.connect(self.pbtnSelectProjectPath,SIGNAL("clicked()"),self.selectProjectPath)
        self.connect(self.pbtnClear,SIGNAL("clicked()"),self.Clear)
        self.connect(self.pbtnClose,SIGNAL("clicked()"),self.close)
        pnlMain = MWidget(self)
        tabwTabs = MTabWidget()
        vblMain = MVBoxLayout(pnlMain)
        pnlMain2 = MWidget(tabwTabs)
        vblMain2 = MVBoxLayout(pnlMain2)
        HBox = MHBoxLayout()
        HBox.addWidget(self.lePathOfProject)
        HBox.addWidget(self.pbtnSelectProjectPath)
        HBox1 = MHBoxLayout()
        HBox1.addWidget(self.pbtnClear)
        HBox1.addWidget(self.pbtnClose)
        vblMain2.addWidget(lblPleaseSelect)
        vblMain2.addLayout(HBox)
        vblMain2.addStretch(1)
        vblMain2.addLayout(HBox1)
        tabwTabs.addTab(pnlMain2, translate("Cleaner", "Clear"))
        tabwTabs.addTab(wOptionsPanel, translate("Cleaner", "Quick Options"))
        vblMain.addWidget(tabwTabs)
        if MyDialogType=="MDialog":
            if Universals.isActivePyKDE4==True:
                self.setMainWidget(pnlMain)
            else:
                self.setLayout(vblMain)
        elif MyDialogType=="MMainWindow":
            self.setCentralWidget(pnlMain)
            moveToCenter(self)
        self.setWindowTitle(translate("Cleaner", "Cleaner"))
        self.setWindowIcon(MIcon("Images:clear.png"))
        self.show()
                        
    def closeEvent(self, _event):
        MApplication.setQuitOnLastWindowClosed(True)
    
    def Clear(self):
        try:
            Universals.isCanBeShowOnMainWindow = False
            answer = Dialogs.ask(translate("Cleaner", "Your Files Will Be Removed"),
                    str(translate("Cleaner", "The files in the \"%s\" folder will be cleared according to the criteria you set.<br>"+
                    "This action will delete the files completely, without any chance to recover.<br>"+
                    "Are you sure you want to perform the action?")) % Organizer.getLink(Organizer.getLink(str(self.lePathOfProject.text()))))
            if answer==Dialogs.Yes:
                if InputOutputs.IA.clearCleaningDirectory(str(self.lePathOfProject.text()), True, True):
                    Dialogs.show(translate("Cleaner", "Directory Is Cleared"),
                                str(translate("Cleaner", "This directory is cleared : \"%s\"")) % Organizer.getLink(str(self.lePathOfProject.text())))
            Universals.isCanBeShowOnMainWindow = True
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 

    def selectProjectPath(self):
        try:
            ProjectPath = MFileDialog.getExistingDirectory(self,
                            translate("Cleaner", "Please Select Directory"),self.lePathOfProject.text())
            if ProjectPath!="":
                self.lePathOfProject.setText(ProjectPath)
        except:
            import ReportBug
            error = ReportBug.ReportBug()
            error.show() 
    
    
                
